 /**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  
#ifndef __CAN_H__
#define __CAN_H__

#include "fm33fg0xxa_fl.h"
#include "fm33fg0xxa_fl_gpio.h"
#include "mf_config.h"

typedef struct
{
    uint32_t StdId;  /*!< Specifies the standard identifier.
                        This parameter can be a value between 0 to 0x7FF. */

    uint32_t ExtId;  /*!< Specifies the extended identifier.
                        This parameter can be a value between 0 to 0x1FFFFFFF. */

    uint8_t DLC;     /*!< Specifies the length of the frame that will be
                        transmitted. This parameter can be a value between
                        0 to 8 */

    uint8_t Data[8]; /*!< Contains the data to be transmitted. It ranges from 0
                        to 0xFF. */
    volatile uint8_t FullFlg;   /* 有数据标记 */
} CanTxMsg;

extern CanTxMsg CAN_TxMessage;               


typedef struct
{
    uint32_t StdId;  /*!< Specifies the standard identifier.
                        This parameter can be a value between 0 to 0x7FF. */

    uint32_t ExtId;  /*!< Specifies the extended identifier.
                        This parameter can be a value between 0 to 0x1FFFFFFF. */

    uint8_t DLC;     /*!< Specifies the length of the frame that will be received.
                        This parameter can be a value between 0 to 8 */

    uint8_t Data[8]; /*!< Contains the data to be received. It ranges from 0 to
                        0xFF. */
    volatile uint8_t FullFlg;   /* 有数据标记 */
} CanRxMsg;
extern CanRxMsg CAN_RxMessage;

#define CAN_TIMEOUT   0x5FFFFU

extern uint8_t Data1[8];
void FL_CAN_BUFFER_Write(uint32_t id, uint32_t len, uint32_t data1, uint32_t data2, CAN_Type *CANx, uint32_t buffer_Chl);
void FL_CAN_FIFO_Read(CAN_Type *CANx);
void CAN_SetMsg(CanTxMsg *TxMesg, uint8_t *Data);
void Clockchange(void);
void FL_CAN_Frame_Send(uint32_t frame_format, CanTxMsg *TxMesg, CAN_Type *CANx, uint32_t buffer_Chl);
int32_t Can_Sleep_Mode_Init(CAN_Type *const CANx);
int32_t Can_Controller_Init(void);
int32_t Can_Controller_DeInit(void);
int32_t CAN_TxMsgFill(uint8_t *data, uint32_t lenth, uint32_t id);
#if (SLEEP_SUPPORT == 1)
void can_sleep_init(void);
void can_awake_init(uint8_t awake_type);
uint8_t can_is_sleep(void);
#endif
#endif

