 /**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  

/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __MF_CONFIG_H
#define __MF_CONFIG_H

#ifdef __cplusplus
extern "C" {
#endif
#include "fm33fg0xxa_fl.h"
#include "stdint.h"
#include "string.h"
    
/* BIT MAP */
#define BIT0            0x00000001UL
#define BIT1            0x00000002UL
#define BIT2            0x00000004UL
#define BIT3            0x00000008UL
#define BIT4            0x00000010UL
#define BIT5            0x00000020UL
#define BIT6            0x00000040UL
#define BIT7            0x00000080UL
#define BIT8            0x00000100UL
#define BIT9            0x00000200UL
#define BIT10           0x00000400UL
#define BIT11           0x00000800UL
#define BIT12           0x00001000UL
#define BIT13           0x00002000UL
#define BIT14           0x00004000UL
#define BIT15           0x00008000UL
#define BIT16           0x00010000UL
#define BIT17           0x00020000UL
#define BIT18           0x00040000UL
#define BIT19           0x00080000UL
#define BIT20           0x00100000UL
#define BIT21           0x00200000UL
#define BIT22           0x00400000UL
#define BIT23           0x00800000UL
#define BIT24           0x01000000UL
#define BIT25           0x02000000UL
#define BIT26           0x04000000UL
#define BIT27           0x08000000UL
#define BIT28           0x10000000UL
#define BIT29           0x20000000UL
#define BIT30           0x40000000UL
#define BIT31           0x80000000UL

typedef union
{
  unsigned char B08;
  struct
  {
    unsigned char bit0:1;
    unsigned char bit1:1;
    unsigned char bit2:1;
    unsigned char bit3:1;
    unsigned char bit4:1;
    unsigned char bit5:1;
    unsigned char bit6:1;
    unsigned char bit7:1;
  }Bit;
}B08_Bit;

union B16_B08_2                        /* 小端存储(51为大端) */
{
    unsigned short    B16;
    unsigned char    B08[2];
};

union S16_B08_2                        /* 小端存储(51为大端) */
{
    unsigned char    B08[2];
    signed short    S16;
};

union B32_B08_2                        /* 小端存储(51为大端) */
{
    unsigned long int B32;
    unsigned char    B08[4];
};

union B16_B08
{
    unsigned char    B08[2];
    /* unsigned int B16; */
};

union S16_B08
{
    unsigned char    B08[2];
    /* unsigned int S16; */
};

union B32_B08
{
    unsigned char    B08[4];
    /* unsigned long int B32; */
};
    
/* 工程配置 */
#define __SYSTEM_CLOCK              (64000000)
#define __SYSTEM_CLOCK_1MHz_PRE     (__SYSTEM_CLOCK/1000000)
#define __SYSTEM_CLOCK_1KHz_PRE     (__SYSTEM_CLOCK/1000)
#define __SYSTEM_CLOCK_10KHz_PRE    (__SYSTEM_CLOCK/10000)

/* 外设时钟相关参数配置 */
#define PLLDB_VALUE                 (__SYSTEM_CLOCK_1MHz_PRE - 1)
#define BSTIM32_AUTORELOAD_VALUE    5000    /* us为单位,时钟源APB=AHB */
#define BSTIM16_AUTORELOAD_VALUE    1000    /* us为单位,时钟源APB=AHB */
#define GPTIM0_AUTORELOAD_VALUE     0x10000 /* us为单位,时钟源APB=AHB */
#define SPI_BAUD                    FL_SPI_BAUDRATE_DIV16 /* 固定APB 16分频 */
#define I2C_BAUDRATE                100000  /* 100K */
#define I2C_BRG                     ((__SYSTEM_CLOCK/(2 * I2C_BAUDRATE)) - 1) /* I2C对应BRG */

#define VERIFY_SYSREG_TIMER         100000
#define VERIFY_CTRLDATA_PARA_TIMER  200000
#define MAIN_LOOP_TIMER_PERIOD      500000

/* 功能配置 */
#define SLEEP_SUPPORT               1   /* 是否支持sleep 0-不支持 1-支持 */
#define EEPROM_SUPPORT              0   /* 是否支持EEprom */

/* 中断优先级配置 */
#define ECCC_RAMP_FEDT_IRQ_PRI      0   /* ECC 存储器访问保护 停振 中断 */
#define SVD_IRQ_PRI                 2   /* SVD 中断 */
#define LIN_SLAVE_UART_IRQ_PRI      1   /* Lin Slave UART中断 */
#define LIN_MASTER_UART_IRQ_PRI     1   /* Lin Master UART中断 */
#define TAU_PGL_IRQ_PRI             2   /* Lin 超时定时器中断 TAU13；TAU07输出PWM中断 */
#define EXTI_DAC_IRQ_PRI            2   /* 休眠时按键中断 */
#define LPTIM_BSTIM_IRQ_PRI         3   /* Lin Master 任务处理中断LPTIM16；ADC序列转换控制中断BSTIM16，BSTIM32 */
#define CAN_IRQ_PRI                 3   /* FSCAN 中断 */
#define CANFD0_IRQ_PRI              3   /* CANFD0 中断 */
#define CANFD1_IRQ_PRI              3   /* CANFD1 中断 */
#define ADC_IRQ_PRI                 3   /* ADC 中断 */
#define DMA_IRQ_PRI                 3   /* DMA 中断 */
#define WKUPx_IRQ_PRI               2   /* WKUPx 中断 */
#define GPTIMx_IRQ_PRI              2   /* GPTIMx 中断 */

/* 按键 */    
#define KEY_GPIO   GPIOE
#define KEY_PIN    FL_GPIO_PIN_6

#define KEY_N      (!(KEY_GPIO->DIN & KEY_PIN))
#define KEY_P      (KEY_GPIO->DIN & KEY_PIN)

/* Can命令码 必须匹配标准数据帧 */
#define CAN_CTRL_REQUEST_CMD_ID     0x17
#define CAN_CTRL_RESPONSE_CMD_ID    0x37
#define CANFD_CTRL_REQUEST_CMD_ID   0x17
#define CANFD_CTRL_RESPONSE_CMD_ID  0x37
    
typedef enum
{
    DAC_TEST = 0x1,
    I2C_TEST = 0x2,
    SPI_TEST = 0x4,
    CODE_FLASH_TEST = 0x8,
    DATA_FLASH_TEST = 0x10
}TEST_TYPE;

#define MESSAGE_FROM_CAN    0
#define MESSAGE_FROM_CANFD  1
#define MESSAGE_FROM_LIN    2

/* Can & Lin 通信数据 */
typedef union
{
    uint8_t data[8];
    struct
    {
        /* PWM频率以及占空比 */
        uint32_t pwm_freq :24;/* Hz为单位，范围40Hz~10KHz中能被10KHz整除的任意频率(非整除会有偏差)，0xFFFFFF维持不变 */
        uint32_t pwm_duty :8;/* Hex, 0~100 0xFF维持不变 */
        
        uint32_t led_status :2;/* 0-关 1-开 其余-维持不变 */
        uint32_t test_ctrl :6;/* 参考 DAC_TEST 定义 */
        uint32_t rsvd :24;/* 保留 */
    }str;
}ctrl_data_type;

typedef struct
{
    ctrl_data_type ctrl_data;
    union B16_B08 crc16;
    uint8_t update_flag; /* 0-无更新 1-有更新 */
    uint8_t message_src; /* 消息来源 0-CAN 1-CANFD 2-Lin */
}app_struct_type;

extern volatile app_struct_type g_app_struct;

/* ============================EEPROM存储定义========================================== */
#define        ee_ctrldata_page        0x00        /* 控制数据页地址 */
    #define        ee_ctrldata_lenth        8

/* ============================DATAFLASH存储定义========================================== */
#define DATA_FLASH_BKUP_PAGE        8       /* 备份page所在位置 */
#define DATA_FLASH_CRC_PAGE         16      /* 带2字节CRC page */

#define        dataflash_ctrldata_page        0x00        /* 控制数据页地址 */
    #define        dataflash_ctrldata_offset        0x00        /* 控制数据页地址偏移 */
    #define        dataflash_ctrldata_lenth        8

/* ============================inline函数定义========================================== */
__STATIC_INLINE uint32_t Enter_Critical( void )    
{
  uint32_t reg;
  reg=__get_PRIMASK();
  __set_PRIMASK(1);
  return reg;
} 
__STATIC_INLINE void Exit_Critical( uint32_t reg )    
{
  __set_PRIMASK(reg);
   return ;
} 
#define START_CRITICAL()    uint32_t reg;
#define ENTER_CRITICAL()    reg = Enter_Critical()
#define EXIT_CRITICAL()     Exit_Critical(reg)
    
/* Exported functions prototypes ---------------------------------------------*/
void GPIO_Get_InitParam(GPIO_Type* GPIOx, uint32_t pin, FL_GPIO_InitTypeDef* GPIO_InitStruct);
void Close_None_GPIO(void);
void AnalogIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t driveStrength );
void InputIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t pull );
void OutputIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t outputType, uint8_t driveStrength );
void AltFunIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t outputType, uint8_t pull,  uint8_t remapPin, uint8_t driveStrength  );
void CloseIO( GPIO_Type* GPIOx, uint32_t PinNum );
void Close_And_ClearIRQ(IRQn_Type IRQn);
void MF_PMU_Init(void);
void MF_Config_Init(void);
void Error_Handler(void);
void assert_failed(uint8_t *file, uint32_t linetmp);
void get_current_ctrl_status(ctrl_data_type *ctrl_data, uint32_t u32Delay_us);
void app_process(void);
#if (SLEEP_SUPPORT == 1)
void key_led_sleep_init(void);
void key_led_awake_init(uint8_t awake_type);
uint8_t key_led_is_sleep(void);
#endif
#ifdef __cplusplus
}
#endif

#endif /* __MF_CONFIG_H */

/************************ (C) COPYRIGHT FMSH *****END OF FILE****/
