 /**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  

/* Includes ------------------------------------------------------------------*/
#include "mf_config.h"
#include "fm33fg0xxa_fl.h"
#include "clock.h"
#include "led.h"
#include "lin.h" 
#include "sleep.h"
#include "svd_interrupt.h"
#include "can.h"
#include "bootloader.h"
#include "canfd.h"
#include "adc.h"
#include "dac.h"
#include "atim.h"
#include "gptim.h"
#include "i2c.h"
#include "spi.h"
#include "flash.h"
#include "ecc.h"
#include "lptim16.h"
#include "verify.h"

/* ADC应用 */
volatile uint16_t CycleCnt = 0;

volatile uint32_t ErrCnt = 0;

extern volatile uint16_t  buf1_AD[2][10];

/* SPI应用 */
#define LENGTH      8
uint8_t rBuffer[LENGTH] = {0};
uint8_t wBuffer[LENGTH] = {0x11, 0x22, 0x33, 0x44, 0x55, 0x66, 0x77, 0x88};


/* 获取引脚配置 */
void GPIO_Get_InitParam(GPIO_Type* GPIOx, uint32_t pin, FL_GPIO_InitTypeDef* GPIO_InitStruct)
{
    uint32_t pinbit = 0x00;
    uint32_t i,temp;

    for (i = 0x00; i < 0x10; i++)
    {
        pinbit = pin & (1 << i);
        if(pinbit > 0) break;                                         
    }
    
    if(i < 0x10)
    {
        pinbit = i;
        GPIO_InitStruct->pin = pin;
        
        /* 输入上下拉配置 */
        if ((GPIOx->PUDEN & pin) && (GPIOx->PUDEN & ((uint32_t)pin << 16)))
        {
            GPIO_InitStruct->pull = FL_GPIO_BOTH_DISABLE;
        }
        else if (GPIOx->PUDEN & pin)
        {
            GPIO_InitStruct->pull = FL_GPIO_PULLUP_ENABLE;
        }
        else if (GPIOx->PUDEN & ((uint32_t)pin << 16))
        {
            GPIO_InitStruct->pull = FL_GPIO_PULLDOWN_ENABLE;
        }
        else
        {
            GPIO_InitStruct->pull = FL_GPIO_BOTH_DISABLE;
        }
        
        /* 输出推挽开漏配置 */
        temp = (GPIOx->ODEN & pin)>>pinbit;                                   
        if(temp == (uint32_t)FL_GPIO_OUTPUT_OPENDRAIN)
        {                                         
            GPIO_InitStruct->outputType = FL_GPIO_OUTPUT_OPENDRAIN;
        }
        else
        {
            GPIO_InitStruct->outputType = FL_GPIO_OUTPUT_PUSHPULL;
        }  
        
        /* 引脚功能配置 */
        temp = (GPIOx->FCR >> (pinbit * 2)) & 0x00000003;                                    
        if(temp == (uint32_t)FL_GPIO_MODE_INPUT)
        {                                         
            GPIO_InitStruct->mode = FL_GPIO_MODE_INPUT;
        }
        else if(temp == (uint32_t)FL_GPIO_MODE_OUTPUT)
        {
            GPIO_InitStruct->mode = FL_GPIO_MODE_OUTPUT;
        }
        else if(temp == (uint32_t)FL_GPIO_MODE_DIGITAL)
        {                                         
            GPIO_InitStruct->mode = FL_GPIO_MODE_DIGITAL;
        }
        else
        {
            GPIO_InitStruct->mode = FL_GPIO_MODE_ANALOG;
        }

        /* 额外数字功能配置 */
        temp = (GPIOx->DFS >> (pinbit * 2)) & 0x00000003;                                    
        if(temp == (uint32_t)FL_GPIO_PINREMAP_FUNCTON0)
        {                                         
            GPIO_InitStruct->remapPin = FL_GPIO_PINREMAP_FUNCTON0;
        }
        else if(temp == (uint32_t)FL_GPIO_PINREMAP_FUNCTON1)
        {
            GPIO_InitStruct->remapPin = FL_GPIO_PINREMAP_FUNCTON1;
        }
        else if(temp == (uint32_t)FL_GPIO_PINREMAP_FUNCTON2)
        {                                         
            GPIO_InitStruct->remapPin = FL_GPIO_PINREMAP_FUNCTON2;
        }
        else
        {
            GPIO_InitStruct->remapPin = FL_GPIO_PINREMAP_FUNCTON3;
        }

        /* 驱动强度配置 */
        temp = (GPIOx->DFS >> (pinbit * 2)) & 0x00000003;                                    
        if(temp == (uint32_t)FL_GPIO_DRIVESTRENGTH_CLOSE)
        {                                         
            GPIO_InitStruct->driveStrength = FL_GPIO_DRIVESTRENGTH_CLOSE;
        }
        else if(temp == (uint32_t)FL_GPIO_DRIVESTRENGTH_X1)
        {
            GPIO_InitStruct->driveStrength = FL_GPIO_DRIVESTRENGTH_X1;
        }
        else if(temp == (uint32_t)FL_GPIO_DRIVESTRENGTH_X3)
        {                                         
            GPIO_InitStruct->driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
        }
        else
        {
            GPIO_InitStruct->driveStrength = FL_GPIO_DRIVESTRENGTH_X4;
        }        
    }  
}

/*  
    模拟功能配置:ADC等 
    开漏，禁止上下拉
    driveStrength参考 FL_GPIO_DRIVESTRENGTH_CLOSE 等
*/
void AnalogIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t driveStrength )
{
    FL_GPIO_InitTypeDef  GPIO_InitStructure;
    FL_GPIO_InitTypeDef  GPIO_InitStructureRun;
    
    GPIO_Get_InitParam(GPIOx, PinNum, &GPIO_InitStructureRun);
    
    if( (GPIO_InitStructureRun.pin                != PinNum) ||
        (GPIO_InitStructureRun.outputType        != FL_GPIO_OUTPUT_OPENDRAIN) ||
        (GPIO_InitStructureRun.pull                != FL_GPIO_BOTH_DISABLE) ||
        (GPIO_InitStructureRun.mode                != FL_GPIO_MODE_ANALOG) ||
        (GPIO_InitStructureRun.remapPin            != FL_GPIO_PINREMAP_FUNCTON0) ||
        (GPIO_InitStructureRun.driveStrength    != driveStrength))
    {
        GPIO_InitStructure.pin = PinNum;
        GPIO_InitStructure.outputType = FL_GPIO_OUTPUT_OPENDRAIN;
        GPIO_InitStructure.pull = FL_GPIO_BOTH_DISABLE;
        GPIO_InitStructure.mode = FL_GPIO_MODE_ANALOG;
        GPIO_InitStructure.remapPin = FL_GPIO_PINREMAP_FUNCTON0;
        GPIO_InitStructure.driveStrength = driveStrength;         
        
        FL_GPIO_Init(GPIOx, &GPIO_InitStructure);        
    }    
}

/* 
    输入IO口配置
    开漏
    pull参考 FL_GPIO_PULLUP_ENABLE 等
*/
void InputIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t pull )
{
    FL_GPIO_InitTypeDef  GPIO_InitStructure;
    FL_GPIO_InitTypeDef  GPIO_InitStructureRun;
    
    GPIO_Get_InitParam(GPIOx, PinNum, &GPIO_InitStructureRun);
    
    if( (GPIO_InitStructureRun.pin                != PinNum) ||
        (GPIO_InitStructureRun.outputType        != FL_GPIO_OUTPUT_OPENDRAIN) ||
        (GPIO_InitStructureRun.pull                != pull) ||
        (GPIO_InitStructureRun.mode                != FL_GPIO_MODE_INPUT) ||
        (GPIO_InitStructureRun.remapPin            != FL_GPIO_PINREMAP_FUNCTON0) ||
        (GPIO_InitStructureRun.driveStrength    != FL_GPIO_DRIVESTRENGTH_X3))    
    {
        GPIO_InitStructure.pin = PinNum;
        GPIO_InitStructure.outputType = FL_GPIO_OUTPUT_OPENDRAIN;
        GPIO_InitStructure.pull = pull;
        GPIO_InitStructure.mode = FL_GPIO_MODE_INPUT;
        GPIO_InitStructure.remapPin = FL_GPIO_PINREMAP_FUNCTON0;
        GPIO_InitStructure.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;         
        
        FL_GPIO_Init(GPIOx, &GPIO_InitStructure);    
    }    
}

/*
    输出IO口配置
    禁止上下拉
    outputType参考 FL_GPIO_OUTPUT_PUSHPULL 等
    driveStrength参考 FL_GPIO_DRIVESTRENGTH_CLOSE 等
*/
void OutputIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t outputType, uint8_t driveStrength )
{
    FL_GPIO_InitTypeDef  GPIO_InitStructure;
    FL_GPIO_InitTypeDef  GPIO_InitStructureRun;
    
    GPIO_Get_InitParam(GPIOx, PinNum, &GPIO_InitStructureRun);

    if( (GPIO_InitStructureRun.pin                != PinNum) ||
        (GPIO_InitStructureRun.outputType        != outputType) ||
        (GPIO_InitStructureRun.pull                != FL_GPIO_BOTH_DISABLE) ||
        (GPIO_InitStructureRun.mode                != FL_GPIO_MODE_OUTPUT) ||
        (GPIO_InitStructureRun.remapPin            != FL_GPIO_PINREMAP_FUNCTON0) ||
        (GPIO_InitStructureRun.driveStrength    != driveStrength))    
    {
        GPIO_InitStructure.pin = PinNum;
        GPIO_InitStructure.outputType = outputType;
        GPIO_InitStructure.pull = FL_GPIO_BOTH_DISABLE;
        GPIO_InitStructure.mode = FL_GPIO_MODE_OUTPUT;
        GPIO_InitStructure.remapPin = FL_GPIO_PINREMAP_FUNCTON0;
        GPIO_InitStructure.driveStrength = driveStrength;
         
        FL_GPIO_Init(GPIOx, &GPIO_InitStructure);        
    }
}


/* 
    数字特殊功能口
    outputType参考 FL_GPIO_OUTPUT_PUSHPULL 等
    pull参考 FL_GPIO_PULLUP_ENABLE 等
    remapPin参考 FL_GPIO_PINREMAP_FUNCTON0 等
    driveStrength参考 FL_GPIO_DRIVESTRENGTH_CLOSE 等
*/
void AltFunIO( GPIO_Type* GPIOx, uint32_t PinNum, uint8_t outputType, uint8_t pull,  uint8_t remapPin, uint8_t driveStrength  )
{                                                                
    FL_GPIO_InitTypeDef  GPIO_InitStructure;
    FL_GPIO_InitTypeDef  GPIO_InitStructureRun;
    
    GPIO_Get_InitParam(GPIOx, PinNum, &GPIO_InitStructureRun);
    
    if( (GPIO_InitStructureRun.pin                != PinNum) ||
        (GPIO_InitStructureRun.outputType        != outputType) ||
        (GPIO_InitStructureRun.pull                != pull) ||
        (GPIO_InitStructureRun.mode                != FL_GPIO_MODE_DIGITAL) ||
        (GPIO_InitStructureRun.remapPin            != remapPin) ||
        (GPIO_InitStructureRun.driveStrength    != driveStrength))    
    {
        GPIO_InitStructure.pin = PinNum;
        GPIO_InitStructure.outputType = outputType;
        GPIO_InitStructure.pull = pull;
        GPIO_InitStructure.mode = FL_GPIO_MODE_DIGITAL;
        GPIO_InitStructure.remapPin = remapPin;
        GPIO_InitStructure.driveStrength = driveStrength;
         
        FL_GPIO_Init(GPIOx, &GPIO_InitStructure);        
    }
}

//关闭IO（输入功能且输入禁止）
//注意SWD接口的PG8,9如果程序改变它们的配置将无法仿真
void CloseIO( GPIO_Type* GPIOx, uint32_t PinNum )
{
    FL_GPIO_DeInit(GPIOx, PinNum);
}

/**
  * @brief  NVIC中断关闭
  * @param  void
  * @retval void
  */
void Close_And_ClearIRQ(IRQn_Type IRQn)
{
    NVIC_DisableIRQ(IRQn);
    NVIC_ClearPendingIRQ(IRQn);	
}

/* Private function prototypes -----------------------------------------------*/

/**
  * @brief  The application entry point.
  * @retval int
  */
void MF_Clock_Init(void)
{
    /* MCU Configuration--------------------------------------------------------*/

    /* Reset of all peripherals, Initializes the Flash interface and the Systick. */

    /* System interrupt init*/

    /* Initialize all configured peripherals */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void MF_SystemClock_Config(void)
{
    
}

void MF_PMU_Init(void)
{
    FL_PMU_SleepInitTypeDef    defaultInitStruct;
    
    /* 当使用到SVD作为唤醒源时，PMU_CR只允许配置Normal Sleep模式 */
    defaultInitStruct.deepSleep       = FL_PMU_SLEEP_MODE_NORMAL;  /* Normal Sleep */ 
    defaultInitStruct.wakeupFrequency = FL_PMU_RCHF_WAKEUP_FREQ_8MHZ;
    defaultInitStruct.wakeupDelay     = FL_PMU_WAKEUP_DELAY_2US;

    (void)FL_PMU_Sleep_Init(PMU, &defaultInitStruct);  
}

void key_led_init(void)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};
    
    /* Key引脚，设置为输入功能 */
    GPIO_InitStruct.pin           = KEY_PIN;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_INPUT;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull          = FL_GPIO_PULLUP_ENABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON0;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;     
    (void)FL_GPIO_Init(KEY_GPIO, &GPIO_InitStruct);

    LED_RED_INIT();
    LED_GREEN_INIT();
    LED_BLUE_INIT();
    
    LED_RED_OFF();
    LED_GREEN_ON();
    LED_BLUE_OFF();
}

void key_process(void)
{
    if (KEY_N)
    {
        FL_DelayMs(100);//软件延时去抖
        if (KEY_N)
        {//短按
            LED_GREEN_TOG();//绿灯翻转
        }
        while (KEY_N);//等待按键释放
    }    
}

/**
  * @brief  配置CAN控制器
  * @param  void
  * @retval void
  */
void MF_Config_Init(void)
{
    app_struct_type temp_app_struct;
    uint32_t primask;
    
    /* 配置唤醒后的工作环境  */
    MF_PMU_Init();
    
    /* 先将外部XTHF分频到1M，再利用PLL锁相环倍频至64M，具体请见初始化参数 */
    (void)SelXTHFToPLL(&CMU_XTHF_Init_Data, &CMU_PLL_Init_Data); 

    /* 用于监控XTHF停振事件和PLL失锁事件，先配置系统时钟，再使能该监控定时器, 10ms周期 */
    LPTIM16_Init();    
    
    /* IWDT休眠冻结 */
    FL_IWDT_EnableFreezeWhileSleep(IWDT);
    
    /* I2C初始化 */
    I2C_Init();
    
    /* 加载存储的参数 */
    #if (EEPROM_SUPPORT == 1)
    Ctrldata_EEProm_Verify();
    #endif
    Ctrldata_DataFlash_Verify();    
    
    /* 按键及指示灯初始化 */
    key_led_init();
    
    /* LIN初始化 */
    lin_init();
    
    /* CAN初始化 */
    (void)Can_Controller_Init();
    CANFD_Init(CANFD0);
    
    /* ADC初始化 */
    AdcInit();
    
    CapCaptureInit();

    /* BSTIM32初始化 */
    BSTIM32_Init();    
    
    /* DAC初始化 */
    DAC_Init();
    
    /* 输出比较初始化 */
    primask = __get_PRIMASK();
    __disable_irq();
    memcpy((void *)&temp_app_struct, (void *)&g_app_struct, sizeof(app_struct_type));
    __set_PRIMASK(primask);
    ATIM_PWM_Init(temp_app_struct.ctrl_data.str.pwm_freq, temp_app_struct.ctrl_data.str.pwm_duty);
    
    /* 输入捕获初始化 */
    GPTIM_IC_Init();
    
    /*初始化SPI*/
    MF_SPI_Init(SPI3);

////    /* RAM ECC纠错中断初始化 */
////    ECCInterruptInRAMInit();
////    /* Code Flash ECC纠错中断初始化*/
////    ECCInterruptInCodeFlashInit();
////    /* Data Flash ECC纠错中断初始化 */
////    ECCInterruptInDataFlashInit();   
}

void get_current_ctrl_status(ctrl_data_type *ctrl_data, uint32_t u32Delay_us)
{
    uint32_t frequency;
    uint8_t duty;
    
    /* 获取当前Led状态 */
    if (LED_GREEN_GPIO->DO & LED_GREEN_PIN)
    {
        ctrl_data->str.led_status = 0;
    }
    else
    {
        ctrl_data->str.led_status = 1;
    }
    
    /* 获取当前PWM状态 */
    if (0 == u32Delay_us)
    {
        u32Delay_us += 1;
    }
    
    while (get_pwm_status(&frequency, &duty) && u32Delay_us)
    {
        if (--u32Delay_us)
        {
            FL_DelayUs(1);
        }
    }
    
    if (0 == u32Delay_us)
    {
        ctrl_data->str.pwm_freq = 0;
        ctrl_data->str.pwm_duty = 0;        
    }
    else
    {
        ctrl_data->str.pwm_freq = frequency;
        ctrl_data->str.pwm_duty = duty;         
    }
    
//    ctrl_data->str.test_ctrl = 0;
}

void app_process(void)
{
    app_struct_type temp_app_struct;
    uint32_t primask;
    
    temp_app_struct.update_flag = 0;
    
    /* 中断全局变量，竞态保护 */
    primask = __get_PRIMASK();
    __disable_irq();
    if (g_app_struct.update_flag)
    {
        memcpy((void *)&temp_app_struct, (void *)&g_app_struct, sizeof(app_struct_type));
        
        g_app_struct.update_flag = 0;
    }
    __set_PRIMASK(primask);
    
    if (temp_app_struct.update_flag)
    {
        /* 存储新的参数到EEprom和DataFlash */
        #if (EEPROM_SUPPORT == 1)
        WriteE2WithBackup(I2C1, ee_ctrldata_page, (uint8_t *)temp_app_struct.ctrl_data.data, ee_ctrldata_lenth );
        #endif
        WriteDataFlashWithBackup( dataflash_ctrldata_page, dataflash_ctrldata_offset, (uint8_t *)temp_app_struct.ctrl_data.data, dataflash_ctrldata_lenth );
        
        /* Led控制 */
        if (1 == temp_app_struct.ctrl_data.str.led_status)
        {
            LED_GREEN_ON();
        }
        else if (0 == temp_app_struct.ctrl_data.str.led_status)
        {
            LED_GREEN_OFF();
        }
        
        /* PWM控制 */
        if ((0xFFFFFF != temp_app_struct.ctrl_data.str.pwm_duty) || 
            (0xFF != temp_app_struct.ctrl_data.str.pwm_freq))
        {
            set_pwm_para(temp_app_struct.ctrl_data.str.pwm_freq, temp_app_struct.ctrl_data.str.pwm_duty);
            clear_pwm_update_flag();
        }
        
        /* test控制 */
        if (temp_app_struct.ctrl_data.str.test_ctrl & DAC_TEST)
        {
            /* DAC样例测试 */
            TestDAC();
        }
        if (temp_app_struct.ctrl_data.str.test_ctrl & I2C_TEST)
        {
            /* I2C读写测试 */
            (void)I2CWriteAndRead();
        }
        if (temp_app_struct.ctrl_data.str.test_ctrl & SPI_TEST)
        {
            /* SPI读写测试 */
            SpiWriteData(SPI3, wBuffer, LENGTH);
            SpiReadData(SPI3, rBuffer, LENGTH);
        }
        if (temp_app_struct.ctrl_data.str.test_ctrl & CODE_FLASH_TEST)
        {
            /* Code Flash 擦写示例 */
            CodeFlash_RW();
        }
        if (temp_app_struct.ctrl_data.str.test_ctrl & DATA_FLASH_TEST)
        {
            /* Data Flash 擦写示例 */
            DataFlash_RW();
        }
        
        get_current_ctrl_status(&temp_app_struct.ctrl_data, 1000);/* 等待1000us */
        if (MESSAGE_FROM_CAN == temp_app_struct.message_src)
        {
            CAN_TxMsgFill(temp_app_struct.ctrl_data.data, 8, CAN_CTRL_RESPONSE_CMD_ID);
        }
        else if (MESSAGE_FROM_CANFD == temp_app_struct.message_src)
        {
            CANFD_TxMessage.ID = CANFD_CTRL_RESPONSE_CMD_ID;
            CANFD_SetMsg(&CANFD_TxMessage, temp_app_struct.ctrl_data.data);
            memcpy(CANFD_TxMessage.Data.Data8, temp_app_struct.ctrl_data.data, 8);
            FL_CANFD_Frame_Send(FL_CANFD_FORMAT_STANDARD_DATA, BRS_DISABLE, &CANFD_TxMessage, CANFD0);
        }
        else if (MESSAGE_FROM_LIN == temp_app_struct.message_src)
        {
            memcpy(LIN_SLAVE_CTRL_RESPONSE_Data, temp_app_struct.ctrl_data.data, 8);
        }
    }
    
    key_process();
}

#if (SLEEP_SUPPORT == 1)
/**
  * @brief  Key & Led 休眠前初始化
  * @param  void
  * @retval void
  */
void key_led_sleep_init(void)
{
    FL_EXTI_InitTypeDef extiInitStruct = {0};
    FL_EXTI_CommonInitTypeDef extiCommonInitStruct = {0};    
    
    /* 关闭非唤醒外设 */
    
    /* 关闭非唤醒IO口 */
    CloseIO(GPIOB, FL_GPIO_PIN_6);/* 关闭led */
    
    /* 唤醒源配置 */ 

    /* EXTI中断采样时钟选择 */
    extiCommonInitStruct.clockSource = FL_CMU_EXTI_CLK_SOURCE_LSCLK;
    /* 休眠使能外部中断采样 */
    extiCommonInitStruct.EXTIOnSleep = FL_ENABLE;
    (void)FL_EXTI_CommonInit(&extiCommonInitStruct);

    /* 配置 key 引脚外部中断功能 */
    extiInitStruct.extiPinX    = KEY_PIN; 
    /* 使能数字滤波 */
    extiInitStruct.filter      = FL_ENABLE;
    /* 设置触发边沿 */
    extiInitStruct.triggerEdge = FL_GPIO_EXTI_TRIGGER_EDGE_FALLING;  
    (void)FL_EXTI_Init(KEY_GPIO, &extiInitStruct);    

    /* 清除中断标识 */
    FL_GPIO_ClearFlag_EXTI_PE(GPIO, KEY_PIN);

    NVIC_DisableIRQ(EXTI_DAC_IRQn);
    NVIC_SetPriority(EXTI_DAC_IRQn, EXTI_DAC_IRQ_PRI); 
    NVIC_EnableIRQ(EXTI_DAC_IRQn);
}

/**
  * @brief  Key & Led 唤醒后初始化
  * @param  0-普通 1-作为本次唤醒的源
  * @retval void
  */
void key_led_awake_init(uint8_t awake_type)
{
    /* 打开非唤醒IO */
    OutputIO( GPIOB, FL_GPIO_PIN_6, FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_DRIVESTRENGTH_X3 );    //PB6;//指示灯
    
    /* 任务初始化 */

    /* 唤醒源处理 */
    /* 关闭中断 */
    FL_GPIO_SetTriggerEdgePE(GPIO, KEY_PIN, FL_GPIO_EXTI_TRIGGER_EDGE_DISABLE);
    Close_And_ClearIRQ(EXTI_DAC_IRQn);
    
    /* 非唤醒外设处理 */
}

/**
  * @brief  Key & Led sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t key_led_is_sleep(void)
{
    if (1)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

/**
  * @brief  GPIO中断函数
  * @param  void
  * @retval None
  */
void EXTI_DAC_IRQHandler(void)
{
    uint32_t triggerEdge;
    uint32_t flag_EXTI;
    
    /* key 引脚外部中断处理 */
    triggerEdge = FL_GPIO_GetTriggerEdgePE(GPIO, KEY_PIN);
    flag_EXTI = FL_GPIO_IsActiveFlag_EXTI_PE(GPIO, KEY_PIN);
    
    if ((FL_GPIO_EXTI_TRIGGER_EDGE_DISABLE != triggerEdge) && (0x01 == flag_EXTI))
    {   /* 清除中断标识 */
        FL_GPIO_ClearFlag_EXTI_PE(GPIO, KEY_PIN);
        
        /* 唤醒处理 */
        key_led_awake_init(0);
    }
}
#endif


/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
    /* USER CODE BEGIN Error_Handler_Debug */
    /* User can add his own implementation to report the HAL error return state */

    /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN Assert_Failed */
    /* User can add his own implementation to report the file name and line number,
       tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END Assert_Failed */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT FMSH *****END OF FILE****/
