 /**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  

#include "atim.h"
#include "main.h"

/**
  * @brief  ATIM初始化
  * @param  void
  * @retval void
  */
void ATIM_PWM_Init(uint32_t frequency, uint8_t duty)
{
    FL_ATIM_InitTypeDef        InitStructer1;
    FL_ATIM_OC_InitTypeDef     InitStructer2;
    FL_ATIM_BDTR_InitTypeDef   InitStructer3;
    FL_GPIO_InitTypeDef         GPIO_InitStruct = {0};
    uint32_t temp_prescaler, temp_compare;
    
    if (((frequency < 25) || (frequency > 10000)) ||
        duty > 100)
    {
        return;
    }
    temp_prescaler = 10000/frequency - 1;
    if (duty == 0)
    {
        temp_compare = 0;        
    }
    else if (duty == 100)
    {
        temp_compare = __SYSTEM_CLOCK_10KHz_PRE;
    }
    else
    {
        temp_compare = duty * __SYSTEM_CLOCK_10KHz_PRE/100 - 1;
    }    

    /*----------------------------------GPIO结构体初始化-------------------------------------*/
    GPIO_InitStruct.pin        = FL_GPIO_PIN_4;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull       = FL_DISABLE;
    GPIO_InitStruct.remapPin   = FL_GPIO_PINREMAP_FUNCTON0;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;		
    FL_GPIO_Init(GPIOB, &GPIO_InitStruct); /* CH1 */

    /*----------------------------------时基结构体初始化------------------------------------*/
    InitStructer1.clockSource           = FL_CMU_ATIM_CLK_SOURCE_APBCLK;  /* 时钟源选择APB1 */
    InitStructer1.prescaler             = temp_prescaler;                 /* 分频到目标频率 */
    InitStructer1.counterMode           = FL_ATIM_COUNTER_DIR_UP;         /* 向上计数 */
    InitStructer1.autoReload            = __SYSTEM_CLOCK_10KHz_PRE-1;     /* 自动重装载值10000 */
    InitStructer1.clockDivision         = FL_ATIM_CLK_DIVISION_DIV1;      /* 死区和滤波分频 */
    InitStructer1.repetitionCounter     = 0;                              /* 重复计数 */
    InitStructer1.autoReloadState       = FL_ENABLE;                      /* 预装载preload使能 */
    FL_ATIM_Init(ATIM, &InitStructer1);

    /*------------------------------输出比较结构体初始化------------------------------------*/
    InitStructer2.OCMode       = FL_ATIM_OC_MODE_PWM1;                    /* 输出比较模式PWM1 */
    InitStructer2.OCETRFStatus = FL_DISABLE;                              /* OC1REF不受ETR影响 */
    InitStructer2.OCFastMode   = FL_DISABLE;                              /* 关闭快速使能 */
    InitStructer2.compareValue = temp_compare;                            /* 比较值1000 */
    InitStructer2.OCPolarity   = FL_ATIM_OC_POLARITY_NORMAL;              /* OC 高电平有效 */
    InitStructer2.OCPreload    = FL_DISABLE;                              /* OC preload 无效 */
    InitStructer2.OCIdleState  = FL_ATIM_OC_IDLE_STATE_HIGH;              /* OC IDLE时为高电平 */
    InitStructer2.OCNIdleState = FL_ATIM_OCN_IDLE_STATE_HIGH;             /* OCN IDLE时为高电平 */
    InitStructer2.OCNPolarity  = FL_ATIM_OCN_POLARITY_NORMAL;             /* OCN 高电平有效 */
    InitStructer2.OCNState     = FL_DISABLE;                              /* 禁能互补输出通道 */
    InitStructer2.OCState      = FL_ENABLE;                               /* 使能正向输出通道 */
    FL_ATIM_OC_Init(ATIM, FL_ATIM_CHANNEL_1, &InitStructer2);

    /*------------------------------刹车死区结构体初始化-----------------------------------*/
    InitStructer3.deadTime            = 100;                              /* 死区时间配置 */
    InitStructer3.lockLevel           = FL_ATIM_LOCK_LEVEL_OFF;           /* 寄存器写保护关闭 */
    InitStructer3.OSSRState           = FL_ATIM_OSSR_DISABLE;             /* OSSR=0 */
    InitStructer3.OSSIState           = FL_ATIM_OSSI_DISABLE;             /* OSSI=0 */
    InitStructer3.breakFilter         = FL_ATIM_BREAK_FILTER_DIV1;        /* 刹车信号滤波配置 */
    InitStructer3.breakPolarity       = FL_ATIM_BREAK_POLARITY_HIGH;      /* 刹车信号高电平有效 */
    InitStructer3.automaticOutput     = FL_DISABLE;                       /* MOE由软件置位 */
    InitStructer3.gatedBrakeSignal_1  = FL_ATIM_BREAK1_GATE_AUTO;         /* 刹车信号1不门控 */
    InitStructer3.gatedBrakeSignal_2  = FL_ATIM_BREAK2_GATE_AUTO;         /* 刹车信号2不门控 */
    InitStructer3.brakeSignalCombined = FL_ATIM_BREAK_COMBINATION_OR;     /* 刹车信号1和2或 */
    InitStructer3.breakState          = FL_DISABLE;                       /* 刹车信号禁止 */
    FL_ATIM_BDTR_Init(ATIM, &InitStructer3);

    FL_ATIM_Enable(ATIM);         /* 使能定时器 */

    FL_ATIM_EnableALLOutput(ATIM);/* 主控输出使能 */
}

/**
  * @brief  ATIM模块复位
  * @param  void
  * @retval void
  */
void ATIM_DeInit(void)
{
    /* 反初始化ATIM寄存器 */
    FL_ATIM_DeInit(ATIM);

    /* 清除NVIC Pending标志位 */
    NVIC_ClearPendingIRQ(ATIM_IRQn);

    /* 关闭NVIC中断 */
    NVIC_DisableIRQ(ATIM_IRQn);
}

/* 能被10KHz整分频的频率 */
void set_pwm_para(uint32_t frequency, uint8_t duty)
{
    FL_ATIM_InitTypeDef        InitStructer1;
    FL_ATIM_OC_InitTypeDef     InitStructer2;
    uint32_t temp_prescaler, temp_compare;
    
    if (((frequency < 25) || (frequency > 10000)) ||
        duty > 100)
    {
        return;
    }
    temp_prescaler = 10000/frequency - 1;
    if (duty == 0)
    {
        temp_compare = 0;
    }
    else if (duty == 100)
    {
        temp_compare = __SYSTEM_CLOCK_10KHz_PRE;
    }
    else
    {
        temp_compare = duty * __SYSTEM_CLOCK_10KHz_PRE/100 - 1;
    }
    
    FL_ATIM_Disable(ATIM);         /* 禁能定时器 */
    
    /*----------------------------------时基结构体初始化------------------------------------*/
    InitStructer1.clockSource           = FL_CMU_ATIM_CLK_SOURCE_APBCLK;  /* 时钟源选择APB2 */
    InitStructer1.prescaler             = temp_prescaler;                 /* 分频到目标频率 */
    InitStructer1.counterMode           = FL_ATIM_COUNTER_DIR_UP;         /* 向上计数 */
    InitStructer1.autoReload            = __SYSTEM_CLOCK_10KHz_PRE-1;     /* 自动重装载值10000 */
    InitStructer1.clockDivision         = FL_ATIM_CLK_DIVISION_DIV1;      /* 死区和滤波分频 */
    InitStructer1.repetitionCounter     = 0;                              /* 重复计数 */
    InitStructer1.autoReloadState       = FL_ENABLE;                      /* 预装载preload使能 */
    FL_ATIM_Init(ATIM, &InitStructer1);

    /*------------------------------输出比较结构体初始化------------------------------------*/
    InitStructer2.OCMode       = FL_ATIM_OC_MODE_PWM1;                    /* 输出比较模式PWM1 */
    InitStructer2.OCETRFStatus = FL_DISABLE;                              /* OC1REF不受ETR影响 */
    InitStructer2.OCFastMode   = FL_DISABLE;                              /* 关闭快速使能 */
    InitStructer2.compareValue = temp_compare;                            /* 比较值 */
    InitStructer2.OCPolarity   = FL_ATIM_OC_POLARITY_NORMAL;              /* OC 高电平有效 */
    InitStructer2.OCPreload    = FL_DISABLE;                              /* OC preload 无效 */
    InitStructer2.OCIdleState  = FL_ATIM_OC_IDLE_STATE_HIGH;              /* OC IDLE时为高电平 */
    InitStructer2.OCNIdleState = FL_ATIM_OCN_IDLE_STATE_HIGH;             /* OCN IDLE时为高电平 */
    InitStructer2.OCNPolarity  = FL_ATIM_OCN_POLARITY_NORMAL;             /* OCN 高电平有效 */
    InitStructer2.OCNState     = FL_DISABLE;                              /* 禁能互补输出通道 */
    InitStructer2.OCState      = FL_ENABLE;                               /* 使能正向输出通道 */
    FL_ATIM_OC_Init(ATIM, FL_ATIM_CHANNEL_1, &InitStructer2);    
  
    FL_ATIM_Enable(ATIM);         /* 使能定时器 */   
}    

#if (SLEEP_SUPPORT == 1)
/**
  * @brief  ATIM休眠前初始化
  * @param  void
  * @retval void
  */
void atim_sleep_init(void)
{
    /* 关闭非唤醒外设 */
    FL_ATIM_Disable(ATIM);         /* 禁能定时器 */
    
    /* 关闭非唤醒IO口 */
    CloseIO(GPIOB, FL_GPIO_PIN_4);
    
    /* 唤醒源配置 */
}

/**
  * @brief  ATIM唤醒后初始化
  * @param  0-普通 1-作为本次唤醒的源
  * @retval void
  */
void atim_awake_init(uint8_t awake_type)
{
    /* 打开非唤醒IO */
    AltFunIO( GPIOB, FL_GPIO_PIN_4,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 ); //PB4;//ATIM PWM输出  
    
    /* 任务初始化 */

    /* 唤醒源处理 */
    
    /* 非唤醒外设处理 */
    FL_ATIM_Enable(ATIM);         /* 使能定时器 */   
}

/**
  * @brief  ATIM sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t atim_is_sleep(void)
{
    if (1)//随时可进入sleep
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
#endif
