 /**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  

#include "dac.h"
#include "mf_config.h"

uint16_t DACdata[2] = {1000,500};

/**
  * @brief  DAC初始化
  * @param  void
  * @retval void
  */
void DAC_Init(void)
{
    FL_GPIO_InitTypeDef    GPIO_InitStruct = {0};
    FL_DAC_InitTypeDef     DAC_InitStruct = {0};

    /* 配置引脚为模拟功能 */
    /* DAC_OUT引脚 PC5 */
    GPIO_InitStruct.pin        = FL_GPIO_PIN_5;
    GPIO_InitStruct.mode       = FL_GPIO_MODE_ANALOG;
    GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.pull       = FL_GPIO_BOTH_DISABLE;
    GPIO_InitStruct.remapPin   = FL_GPIO_PINREMAP_FUNCTON0;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    FL_GPIO_Init( GPIOC, &GPIO_InitStruct );

    /* DAC寄存器配置 */
    DAC_InitStruct.triggerMode   = FL_DISABLE;           /* 触发模式使能配置 */
    DAC_InitStruct.triggerSource = FL_DAC_TRGI_SOFTWARE; /* 触发源配置 */
    DAC_InitStruct.switchMode    = FL_ENABLE;            /* 输出开关配置 */
    FL_DAC_Init(DAC,&DAC_InitStruct);
}

/**
  * @brief  DAC测试
  * @param  void
  * @retval void
  */
void TestDAC(void)
{
    /* 使能DAC模块 */
    FL_DAC_Enable(DAC);
    /* 写入DHR数据 */
    FL_DAC_WriteData(DAC,1023);
    /* 写入DHR数据 */
    FL_DAC_WriteData(DAC,511);
}

/**
  * @brief  DAC模块复位
  * @param  void
  * @retval void
  */
void DAC_DeInit(void)
{
    /* 复位清零DAC寄存器 */
    FL_DAC_DeInit(DAC);

    /* 关闭DAC总线时钟 */
    FL_CMU_DisableGroup2BusClock(FL_CMU_GROUP2_BUSCLK_DAC);

    /* 清除中断挂起 */
    NVIC_ClearPendingIRQ(EXTI_DAC_IRQn);

    /* 关闭NVIC中断 */
    NVIC_DisableIRQ(EXTI_DAC_IRQn);
}

#if (SLEEP_SUPPORT == 1)
/**
  * @brief  DAC休眠前初始化
  * @param  void
  * @retval void
  */
void dac_sleep_init(void)
{
    /* 关闭非唤醒外设 */
    FL_DAC_Disable(DAC);  /* 禁能DAC */
    
    /* 关闭非唤醒IO口 */
    CloseIO(GPIOC, FL_GPIO_PIN_5);
    
    /* 唤醒源配置 */
}

/**
  * @brief  DAC唤醒后初始化
  * @param  0-普通 1-作为本次唤醒的源
  * @retval void
  */
void dac_awake_init(uint8_t awake_type)
{
    /* 打开非唤醒IO */
    AnalogIO( GPIOC, FL_GPIO_PIN_5,  FL_GPIO_DRIVESTRENGTH_X3 );    //PC5;//DAC  
    
    /* 任务初始化 */

    /* 唤醒源处理 */
    
    /* 非唤醒外设处理 */
    FL_DAC_Enable(DAC);  /* 使能DAC */
}

/**
  * @brief  dac sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t dac_is_sleep(void)
{
    if (1)//随时可进入sleep
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
#endif
