 /**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  

#include "ecc.h"
#include "mf_config.h"

volatile uint32_t RAMEccErrorAdd = 0x0;
volatile uint32_t CodeFlashEccErrorAdd = 0x0;
volatile uint32_t DataFlashEccErrorAdd = 0x0;

/**
  * @brief  ECC纠错中断、CMU系统时钟选择错误中断、存储器访问保护中断
  * @param  void
  * @retval None
*/
void ECCC_RAMP_IRQHandler(void)
{
    uint32_t RAMEccIT = 0,       RAMCorrectionFlag = 0;
    uint32_t DataFlashEccIT = 0, DataFlashCorrectionFlag = 0;
    uint32_t CodeFlashEccIT = 0, CodeFlashCorrectionFlag = 0;

    /* RAM ECC 纠错处理 */
    RAMEccIT = FL_FLASH_IsEnabledIT_ECCInterruptInRAM(FLASH);
    RAMCorrectionFlag = FL_FLASH_IsActiveFlag_ECCCorrectionInRAM(FLASH);
    if ((RAMEccIT == 0x01UL) && (RAMCorrectionFlag == 0x01UL))
    {   /* 清除RAM ECC纠错标志 */
        FL_FLASH_ClearFlag_ECCCorrectionInRAM(FLASH);
        /* 客户安全处理 */
        /* ...... */
    }
    
    /* Data Flash ECC 纠错处理 */
    DataFlashEccIT = FL_FLASH_IsEnabledIT_ECCInterruptInDataFlash(FLASH);
    DataFlashCorrectionFlag = FL_FLASH_IsActiveFlag_ECCCorrectionInDataFlash(FLASH);
    if ((DataFlashEccIT == 0x01UL) && (DataFlashCorrectionFlag == 0x01UL))
    {   /* 清除Data Flash ECC纠错标志 */
        FL_FLASH_ClearFlag_ECCCorrectionInDataFlash(FLASH);
        /* 客户安全处理 */
        /* ...... */
    }
    
    /* Code Flash ECC 纠错处理 */
    CodeFlashEccIT = FL_FLASH_IsEnabledIT_ECCInterruptInCodeFlash(FLASH);
    CodeFlashCorrectionFlag = FL_FLASH_IsActiveFlag_ECCCorrectionInCodeFlash(FLASH);
    if ((CodeFlashEccIT == 0x01UL) && (CodeFlashCorrectionFlag == 0x01UL) )
    {   /* 清除Code Flash ECC纠错标志 */
        FL_FLASH_ClearFlag_ECCCorrectionInCodeFlash(FLASH);
        /* 客户安全处理 */
        /* ...... */
    }
}


/**
  * @brief    RAM ECC纠错中断初始化
  * @param    void
  * @retval   None
  */
void ECCInterruptInRAMInit(void)
{
    FL_NVIC_ConfigTypeDef    InterruptConfigStruct;
    #if (0)
    /* 清除RAM ECC纠错标志 */
    FL_FLASH_ClearFlag_ECCCorrectionInRAM(FLASH);
    #endif
    /* 使能RAM ECC纠错中断 */
    FL_FLASH_EnableIT_ECCInterruptInRAM(FLASH);
    
    /* RAM ECC纠错中断 */
    InterruptConfigStruct.preemptPriority = ECCC_RAMP_FEDT_IRQ_PRI;
    FL_NVIC_Init(&InterruptConfigStruct, ECCC_RAMP_FDET_IRQn);
}



/**
  * @brief    Code Flash ECC纠错中断初始化
  * @param    void
  * @retval   None
  */
void ECCInterruptInCodeFlashInit(void)
{
    FL_NVIC_ConfigTypeDef    InterruptConfigStruct;
    #if (0)
    /* 清除Code Flash ECC纠错标志 */
    FL_FLASH_ClearFlag_ECCCorrectionInCodeFlash(FLASH);
    #endif
    /* 使能Code Flash ECC纠错中断 */
    FL_FLASH_EnableIT_ECCInterruptInCodeFlash(FLASH);
    
    /* Code Flash ECC纠错中断 */
    InterruptConfigStruct.preemptPriority = ECCC_RAMP_FEDT_IRQ_PRI;
    FL_NVIC_Init(&InterruptConfigStruct, ECCC_RAMP_FDET_IRQn);
}


/**
  * @brief    Data Flash ECC纠错中断初始化
  * @param    void
  * @retval   None
  */
void ECCInterruptInDataFlashInit(void)
{
    FL_NVIC_ConfigTypeDef    InterruptConfigStruct;
    #if (0)
    /* 清除Data Flash ECC纠错标志 */
    FL_FLASH_ClearFlag_ECCCorrectionInDataFlash(FLASH);
    #endif
    /* 使能Data Flash ECC纠错中断 */
    FL_FLASH_EnableIT_ECCInterruptInDataFlash(FLASH);
    
    /* Data Flash ECC纠错中断 */
    InterruptConfigStruct.preemptPriority = ECCC_RAMP_FEDT_IRQ_PRI;
    FL_NVIC_Init(&InterruptConfigStruct, ECCC_RAMP_FDET_IRQn);
}

