 /**    
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  
#include "gptim.h"
#include "mf_config.h"

static uint8_t   update_flag = 0;
static uint32_t  IC1Value = 0;
static uint32_t  IC2Value = 0;
static uint32_t  DutyCycle = 0;
static uint32_t  Frequency = 0;
static uint32_t  filter_count = 0;
static uint32_t  count = 0;


/**
* @brief  GPTIM中断服务函数
  * @param  void
  * @retval void
  */
void GPTIMX_IRQHandler(void)
{
    if((FL_GPTIM_IsEnabledIT_CC(GPTIM0, FL_GPTIM_CHANNEL_1)) \
      &&FL_GPTIM_IsActiveFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_1))
    {
        count++;
        if(count == 2)
        {
            IC1Value = FL_GPTIM_ReadCompareCH1(GPTIM0);
            IC2Value = FL_GPTIM_ReadCompareCH2(GPTIM0);
            FL_GPTIM_Disable(GPTIM0);
            DutyCycle = 100 * (IC2Value+1) / (IC1Value+1);  /* 占空比 */
            Frequency = (__SYSTEM_CLOCK / __SYSTEM_CLOCK_1MHz_PRE / (IC1Value+1));         /* 频率,主频8M，8分频 */
            count =0;
            FL_GPTIM_Enable(GPTIM0);
            
            filter_count++;
            if (filter_count >= 2)
            {
                filter_count = 0;
                update_flag = 1;                
            }
        }
      
        FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    }
    if((FL_GPTIM_IsEnabledIT_CC(GPTIM0, FL_GPTIM_CHANNEL_2)) \
      &&FL_GPTIM_IsActiveFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_2))
    {
        
        FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    }
}


/**
  * @brief  GPTIM初始化
  * @param  void
  * @retval void
  */
void GPTIM_Init(void)
{
    FL_GPTIM_InitTypeDef      TimerBase_InitStruct;
    FL_NVIC_ConfigTypeDef     InterruptConfigStruct;
  
    /* 定时器频率计算：系统主频/(分频系数*自动重装载值) */
    TimerBase_InitStruct.prescaler       = __SYSTEM_CLOCK_1KHz_PRE-1;    /* 分频系数配置：对应1KHz */
    TimerBase_InitStruct.counterMode     = FL_GPTIM_COUNTER_DIR_UP;      /* 计数模式配置：向上计数 */
    TimerBase_InitStruct.autoReload      = 1000-1;                       /* 自动重装载值配置：1000 */
    TimerBase_InitStruct.clockDivision   = FL_GPTIM_CLK_DIVISION_DIV1;   /* 死区和滤波分频配置：1分频 */
    TimerBase_InitStruct.autoReloadState = FL_ENABLE;                    /* 预装载配置：使能 */
    FL_GPTIM_Init(GPTIM1, &TimerBase_InitStruct);
  
    /* 清除Update标志 */
    FL_GPTIM_ClearFlag_Update(GPTIM1);
    /* 使能Update中断 */
    FL_GPTIM_EnableIT_Update(GPTIM1);

    /* 使能GPTIM模块 */
    FL_GPTIM_Enable(GPTIM1);

    /* 配置NVIC */
    InterruptConfigStruct.preemptPriority = GPTIMx_IRQ_PRI;
    FL_NVIC_Init(&InterruptConfigStruct, GPTIMX_IRQn);
}

/**
  * @brief  GPTIM输出比较初始化
  * @param  void
  * @retval void
  */
void GPTIM_PWM_Init(void)
{
    FL_GPTIM_InitTypeDef        TimerBase_InitStruct;
    FL_GPTIM_OC_InitTypeDef     TimOCInit;
    FL_GPIO_InitTypeDef         GPIO_InitStruct;
  
    /* 初始化引脚功能 */
    GPIO_InitStruct.pin           = FL_GPIO_PIN_10;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    GPIO_InitStruct.pull          = FL_GPIO_PULLUP_ENABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON1;
  
    FL_GPIO_Init(GPIOB, &GPIO_InitStruct);

    /*****************************/
  
    /* 定时器频率计算：系统主频/(分频系数*自动重装载值) */
    TimerBase_InitStruct.prescaler       = __SYSTEM_CLOCK_1KHz_PRE-1;    /* 分频系数配置：1Hz */
    TimerBase_InitStruct.counterMode     = FL_GPTIM_COUNTER_DIR_UP;      /* 计数模式配置：向上计数 */
    TimerBase_InitStruct.autoReload      = 1000-1;                       /* 自动重装载值配置：1000 */
    TimerBase_InitStruct.clockDivision   = FL_GPTIM_CLK_DIVISION_DIV1;   /* 死区和滤波分频配置：1分频 */
    TimerBase_InitStruct.autoReloadState = FL_ENABLE;                    /* 预装载配置：使能 */
    FL_GPTIM_Init(GPTIM0, &TimerBase_InitStruct);
    
    /* 占空比计算：自动重装载值/比较值 */
    TimOCInit.OCMode       = FL_GPTIM_OC_MODE_PWM2;                      /* 输出比较模式PWM1 */
    TimOCInit.OCETRFStatus = FL_DISABLE;                                 /* OC1REF不受ETR影响 */
    TimOCInit.OCFastMode   = FL_DISABLE;                                 /* 关闭快速使能 */
    TimOCInit.CompareValue = 900;                                        /* 比较值900 */
    TimOCInit.OCPolarity   = FL_GPTIM_OC_POLARITY_INVERT;                /* OC正常输出 */
    TimOCInit.OCPreload    = FL_DISABLE;                                 /* OC preload 无效 */
    FL_GPTIM_OC_Init(GPTIM0, FL_GPTIM_CHANNEL_1, &TimOCInit);
    
    FL_GPTIM_Enable(GPTIM0);    /* 使能定时器 */
}




/**
  * @brief  GPTIM输入捕获初始化
  * @param  void
  * @retval void
  */
void GPTIM_IC_Init(void)
{
    FL_GPIO_InitTypeDef         GPIO_InitStruct;
    FL_GPTIM_InitTypeDef        TimerBase_InitStruct;
    FL_GPTIM_IC_InitTypeDef     GPTIM0_InitStruct;
    FL_NVIC_ConfigTypeDef       InterruptConfigStruct;
    FL_GPTIM_SlaveInitTypeDef   Slave_InitStruct;
  
    /* 初始化引脚功能 */
    GPIO_InitStruct.pin           = FL_GPIO_PIN_10;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    GPIO_InitStruct.pull          = FL_GPIO_PULLUP_ENABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON1;
  
    FL_GPIO_Init(GPIOB, &GPIO_InitStruct);
  
    /*****************************/
  
    /* 定时器频率计算：系统主频/(分频系数*自动重装载值) */
    TimerBase_InitStruct.prescaler       = __SYSTEM_CLOCK_1MHz_PRE-1;             /* 分频系数配置：1MHz */
    TimerBase_InitStruct.counterMode     = FL_GPTIM_COUNTER_DIR_UP;               /* 计数模式配置：向上计数 */
    TimerBase_InitStruct.autoReload      = GPTIM0_AUTORELOAD_VALUE - 1;           /* 自动重装载值配置：1000 */
    TimerBase_InitStruct.clockDivision   = FL_GPTIM_CLK_DIVISION_DIV1;            /* 死区和滤波分频配置：1分频 */
    TimerBase_InitStruct.autoReloadState = FL_ENABLE;                             /* 预装载配置：使能 */
    FL_GPTIM_Init(GPTIM0, &TimerBase_InitStruct);
  
    /* 从机模式参数配置 */
    Slave_InitStruct.ITRSourceGroup      = 0;                                     /* 内部触发源配置：0 */
    Slave_InitStruct.slaveMode           = FL_GPTIM_SLAVE_MODE_TRGI_RISE_RST;     /* 从模式配置：复位模式 */
    Slave_InitStruct.triggerSrc          = FL_GPTIM_TIM_TS_TI1FP1;                /* 从模式输入源配置：TI1FP1 */
    Slave_InitStruct.triggerDelay        = FL_DISABLE;                            /* 触发延时配置：禁能 */
    FL_GPTIM_SlaveMode_Init(GPTIM0,&Slave_InitStruct);
  
    /* 输入捕获参数配置 */
    GPTIM0_InitStruct.ICPolarity         = FL_GPTIM_IC_POLARITY_NORMAL;           /* 输入捕获极性配置：上升沿捕获 */
    GPTIM0_InitStruct.ICActiveInput      = FL_GPTIM_CHANNEL_MODE_INPUT_NORMAL;    /* 输入通道模式配置：CC1通道配置为输入，IC1映射至TI1 */
    GPTIM0_InitStruct.ICPrescaler        = FL_GPTIM_IC_PSC_DIV1;                  /* 输入捕获预分频配置：不分频 */
    GPTIM0_InitStruct.ICFilter           = FL_GPTIM_IC_FILTER_DIV1;               /* 输入捕获滤波配置：无滤波*/
    GPTIM0_InitStruct.captureState       = FL_ENABLE;                             /* 输入捕获使能配置：使能*/
    FL_GPTIM_IC_Init(GPTIM0, FL_GPTIM_CHANNEL_1, &GPTIM0_InitStruct);
    
    /* 输入捕获参数配置 */
    GPTIM0_InitStruct.ICPolarity         = FL_GPTIM_IC_POLARITY_INVERT;           /* 输入捕获极性配置：下降沿捕获 */
    GPTIM0_InitStruct.ICActiveInput      = FL_GPTIM_CHANNEL_MODE_INPUT_CROSSOVER; /* 输入通道模式配置：CC1通道配置为输入，IC1映射至TI2 */
    FL_GPTIM_IC_Init(GPTIM0, FL_GPTIM_CHANNEL_2, &GPTIM0_InitStruct);


    /* 清除捕捉溢出标志 */
    FL_GPTIM_ClearFlag_CCOverflow(GPTIM0, FL_GPTIM_CHANNEL_1);
    /* 清除CH1输入捕获中断标志 */
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    /* 使能CH1输入捕获中断 */
    FL_GPTIM_EnableIT_CC(GPTIM0, FL_GPTIM_CHANNEL_1);

    /* 清除CH2输入捕获中断标志 */
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    /* 使能CH2输入捕获中断 */
    FL_GPTIM_EnableIT_CC(GPTIM0, FL_GPTIM_CHANNEL_2);

    /* 配置NVIC */
    InterruptConfigStruct.preemptPriority = GPTIMx_IRQ_PRI;     
    FL_NVIC_Init(&InterruptConfigStruct, GPTIMX_IRQn);

    /* 使能GPTIM0 */
    FL_GPTIM_Enable(GPTIM0);
}





/**
  * @brief  GPTIM1_Channel1 Initialization function
  * @param  void
  * @retval None
  */
void GPTIM1_Channel1_Init(void)
{
    FL_GPIO_InitTypeDef    GPIO_InitStruct;

    FL_GPTIM_OC_InitTypeDef    GPTIM1_InitStruct;

    /* 初始化引脚功能 */
    GPIO_InitStruct.pin           = FL_GPIO_PIN_2;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    GPIO_InitStruct.pull          = FL_GPIO_PULLUP_ENABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON3;
  
    FL_GPIO_Init(GPIOA, &GPIO_InitStruct);

    GPTIM1_InitStruct.OCMode = FL_GPTIM_OC_MODE_PWM2;
    GPTIM1_InitStruct.OCETRFStatus = FL_DISABLE;
    GPTIM1_InitStruct.OCFastMode = FL_DISABLE;
    GPTIM1_InitStruct.CompareValue = 1000;
    GPTIM1_InitStruct.OCPolarity = FL_GPTIM_OC_POLARITY_NORMAL;
    GPTIM1_InitStruct.OCPreload = FL_DISABLE;

    FL_GPTIM_OC_Init(GPTIM1, FL_GPTIM_CHANNEL_1, &GPTIM1_InitStruct);
}

/**
  * @brief  GPTIM1_Channel2 Initialization function
  * @param  void
  * @retval None
  */
void GPTIM1_Channel2_Init(void)
{
    FL_GPIO_InitTypeDef    GPIO_InitStruct;

    FL_GPTIM_OC_InitTypeDef    GPTIM1_InitStruct;

    /* 初始化引脚功能 */
    GPIO_InitStruct.pin           = FL_GPIO_PIN_3;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    GPIO_InitStruct.pull          = FL_GPIO_PULLUP_ENABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON3;
  
    FL_GPIO_Init(GPIOA, &GPIO_InitStruct);

    GPTIM1_InitStruct.OCMode = FL_GPTIM_OC_MODE_PWM2;
    GPTIM1_InitStruct.OCETRFStatus = FL_DISABLE;
    GPTIM1_InitStruct.OCFastMode = FL_DISABLE;
    GPTIM1_InitStruct.CompareValue = 1000;
    GPTIM1_InitStruct.OCPolarity = FL_GPTIM_OC_POLARITY_NORMAL;
    GPTIM1_InitStruct.OCPreload = FL_DISABLE;

    FL_GPTIM_OC_Init(GPTIM1, FL_GPTIM_CHANNEL_2, &GPTIM1_InitStruct);
}

/**
  * @brief  GPTIM1_Channel3 Initialization function
  * @param  void
  * @retval None
  */
void GPTIM1_Channel3_Init(void)
{
    FL_GPIO_InitTypeDef    GPIO_InitStruct;

    FL_GPTIM_OC_InitTypeDef    GPTIM1_InitStruct;

    /* 初始化引脚功能 */
    GPIO_InitStruct.pin           = FL_GPIO_PIN_4;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    GPIO_InitStruct.pull          = FL_GPIO_PULLUP_ENABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON0;
  
    FL_GPIO_Init(GPIOA, &GPIO_InitStruct);

    GPTIM1_InitStruct.OCMode = FL_GPTIM_OC_MODE_PWM2;
    GPTIM1_InitStruct.OCETRFStatus = FL_DISABLE;
    GPTIM1_InitStruct.OCFastMode = FL_DISABLE;
    GPTIM1_InitStruct.CompareValue = 1000;
    GPTIM1_InitStruct.OCPolarity = FL_GPTIM_OC_POLARITY_NORMAL;
    GPTIM1_InitStruct.OCPreload = FL_DISABLE;

    FL_GPTIM_OC_Init(GPTIM1, FL_GPTIM_CHANNEL_3, &GPTIM1_InitStruct);
}

/**
  * @brief  GPTIM1_Channel4 Initialization function
  * @param  void
  * @retval None
  */
void GPTIM1_Channel4_Init(void)
{
    FL_GPIO_InitTypeDef    GPIO_InitStruct;

    FL_GPTIM_OC_InitTypeDef    GPTIM1_InitStruct;

    /* 初始化引脚功能 */
    GPIO_InitStruct.pin           = FL_GPIO_PIN_5;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    GPIO_InitStruct.pull          = FL_GPIO_PULLUP_ENABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON0;
  
    FL_GPIO_Init(GPIOA, &GPIO_InitStruct);

    GPTIM1_InitStruct.OCMode = FL_GPTIM_OC_MODE_PWM2;
    GPTIM1_InitStruct.OCETRFStatus = FL_DISABLE;
    GPTIM1_InitStruct.OCFastMode = FL_DISABLE;
    GPTIM1_InitStruct.CompareValue = 1000;
    GPTIM1_InitStruct.OCPolarity = FL_GPTIM_OC_POLARITY_NORMAL;
    GPTIM1_InitStruct.OCPreload = FL_DISABLE;

    FL_GPTIM_OC_Init(GPTIM1, FL_GPTIM_CHANNEL_4, &GPTIM1_InitStruct);
}

/**
  * @brief  GPTIM初始化
  * @param  void
  * @retval None
  */
void GPTIM_Setup(void)
{
    FL_GPTIM_InitTypeDef    TimerBase_InitStruct;
    FL_DMA_InitTypeDef      DMA_InitStruct;

    DMA_InitStruct.circMode = FL_DISABLE;
    DMA_InitStruct.dataSize = FL_DMA_BANDWIDTH_32B;
    DMA_InitStruct.direction = FL_DMA_DIR_RAM_TO_PERIPHERAL;
    DMA_InitStruct.periphAddress = FL_DMA_PERIPHERAL_FUNCTION8;
    DMA_InitStruct.priority = FL_DMA_PRIORITY_HIGH;
    DMA_InitStruct.memoryAddressIncMode = FL_DMA_MEMORY_INC_MODE_INCREASE;
    DMA_InitStruct.flashAddressIncMode = FL_DMA_CH7_FLASH_INC_MODE_INCREASE;
    FL_DMA_Init(DMA, &DMA_InitStruct, FL_DMA_CHANNEL_6);

    TimerBase_InitStruct.prescaler = 7;
    TimerBase_InitStruct.counterMode = FL_GPTIM_COUNTER_DIR_UP;
    TimerBase_InitStruct.autoReload = 999;
    TimerBase_InitStruct.clockDivision = FL_GPTIM_CLK_DIVISION_DIV1;
    TimerBase_InitStruct.autoReloadState = FL_DISABLE;
    FL_GPTIM_Init(GPTIM1, &TimerBase_InitStruct);

    FL_GPTIM_EnableDMAReq_Update(GPTIM1);

    GPTIM1_Channel1_Init();
    GPTIM1_Channel2_Init();
    GPTIM1_Channel3_Init();
    GPTIM1_Channel4_Init();

    /* 设置GPTIM1的DMA Burst，一次性更新CCR1~CCR4 */
    FL_GPTIM_SetDMABurstAddress(GPTIM1, FL_GPTIM_DMA_BURST_ADDR_CCR1);
    FL_GPTIM_SetDMABurstLength(GPTIM1, FL_GPTIM_DMA_BURST_LENGTH_4);

    /* 启动GPTIM1 */
    FL_GPTIM_Enable(GPTIM1);

    /* 打开DMA全局使能 */
    FL_DMA_Enable(DMA);
}

/**
  * @brief  DMA Burst主循环，输出4路占空比交替
  *         变化的PWM，可以外接LED观察呼吸流水灯现象
  * @param  void
  * @retval None
  */
void GPTIM_Loop(void)
{
    FL_DMA_ConfigTypeDef dma_cfg;
    static uint32_t data[4] = {200, 400, 600, 800};
    static uint32_t dir[4] = {1, 1, 1, 1};

    /* 更新各个LED亮度 */
    int i;
    for(i = 0; i < 4; i++)
    {
        if(dir[i])
        { data[i] += 50; }
        else
        { data[i] -= 50; }

        if(data[i] >= 1000) { dir[i] = 0; }
        else
            if(data[i] <= 50) { dir[i] = 1; }
    }

    /* 启动DMA Burst写入CCR数据 */
    dma_cfg.memoryAddress = (uint32_t)data;
    dma_cfg.transmissionCount = 4 - 1;
    FL_DMA_StartTransmission(DMA, &dma_cfg, FL_DMA_CHANNEL_6);
    while(!FL_DMA_IsActiveFlag_TransferComplete(DMA, FL_DMA_CHANNEL_6));

    /* 延时用于控制呼吸流水灯变化速度 */
    FL_DelayMs(25);
}

void clear_pwm_update_flag(void)
{
    FL_GPTIM_Disable(GPTIM0);
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    count = 0;
    filter_count = 0;
    update_flag = 0;
    FL_GPTIM_Enable(GPTIM0);
}

uint8_t get_pwm_status(uint32_t *frequency, uint8_t *duty)
{
    app_struct_type temp_app_struct;
    uint32_t primask;    
        
    /* 中断全局变量，竞态保护 */
    primask = __get_PRIMASK();
    __disable_irq();
    memcpy((void *)&temp_app_struct, (void *)&g_app_struct, sizeof(app_struct_type));
    __set_PRIMASK(primask);
    
    if (update_flag)
    {
        update_flag = 0;
        *frequency = Frequency;
        *duty = DutyCycle;        
        return 0;
    }
    else if (temp_app_struct.ctrl_data.str.pwm_duty == 0)
    {
        *frequency = 0;
        *duty = 0;        
        return 0; 
    }
    else if (temp_app_struct.ctrl_data.str.pwm_duty == 100)
    {
        *frequency = 0;
        *duty = 100;        
        return 0;
    }
    else
    {
        *frequency = 0xFFFFFF;/* 非法值 */
        *duty = 0xFF;
        return 1;/* 非法值 */
    }
}

#if (SLEEP_SUPPORT == 1)
/**
  * @brief  GPTIM0 休眠前初始化
  * @param  void
  * @retval void
  */
void gptim0_sleep_init(void)
{
    /* 关闭非唤醒外设 */
    FL_GPTIM_Disable(GPTIM0);  /* 禁能定时器 */
    FL_GPTIM_DisableIT_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    FL_GPTIM_DisableIT_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    /* 清除捕捉溢出标志 */
    FL_GPTIM_ClearFlag_CCOverflow(GPTIM0, FL_GPTIM_CHANNEL_1);
    /* 清除CH1输入捕获中断标志 */
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    /* 清除CH2输入捕获中断标志 */
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    Close_And_ClearIRQ(GPTIMX_IRQn);
    
    /* 关闭非唤醒IO口 */
    CloseIO(GPIOB, FL_GPIO_PIN_10);
    
    /* 唤醒源配置 */
}

/**
  * @brief  GPTIM0 唤醒后初始化
  * @param  0-普通 1-作为本次唤醒的源
  * @retval void
  */
void gptim0_awake_init(uint8_t awake_type)
{
    /* 打开非唤醒IO */
    AltFunIO( GPIOB, FL_GPIO_PIN_10, FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_PULLUP_ENABLE, FL_GPIO_PINREMAP_FUNCTON1, FL_GPIO_DRIVESTRENGTH_X3 ); //PB10;//GPTIM输入捕获  
    
    /* 任务初始化 */
    update_flag = 0;
    IC1Value = 0;
    IC2Value = 0;
    DutyCycle = 0;
    Frequency = 0;
    filter_count = 0;
    count = 0;

    /* 唤醒源处理 */
    
    /* 非唤醒外设处理 */
    /* 清除捕捉溢出标志 */
    FL_GPTIM_ClearFlag_CCOverflow(GPTIM0, FL_GPTIM_CHANNEL_1);
    /* 清除CH1输入捕获中断标志 */
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    /* 清除CH2输入捕获中断标志 */
    FL_GPTIM_ClearFlag_CC(GPTIM0, FL_GPTIM_CHANNEL_2);    
    FL_GPTIM_EnableIT_CC(GPTIM0, FL_GPTIM_CHANNEL_1);
    FL_GPTIM_EnableIT_CC(GPTIM0, FL_GPTIM_CHANNEL_2);
    NVIC_EnableIRQ(GPTIMX_IRQn);
    FL_GPTIM_Enable(GPTIM0);  /* 使能定时器 */    
}

/**
  * @brief  GPTIM0 sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t gptim0_is_sleep(void)
{
    if (1)//随时可进入sleep
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
#endif
