/**
  ******************************************************************************
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  
   
#include "spi.h"
#include "mf_config.h"

void MF_SPI_Init(SPI_Type* SPIx)
{
    FL_SPI_InitTypeDef    SPI_InitStruct = {0};
    FL_GPIO_InitTypeDef   GPIO_InitStruct = {0};

    switch((uint32_t)SPIx)
    {
        case SPI0_BASE:
            GPIO_InitStruct.pin = FL_GPIO_PIN_8 | FL_GPIO_PIN_9 | FL_GPIO_PIN_10 |
                                 FL_GPIO_PIN_11;
            GPIO_InitStruct.mode = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull = FL_GPIO_BOTH_DISABLE;
            GPIO_InitStruct.remapPin = FL_GPIO_PINREMAP_FUNCTON0;
            GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
            (void)FL_GPIO_Init(GPIOB, &GPIO_InitStruct);

            break;
        case SPI1_BASE:
            GPIO_InitStruct.pin = FL_GPIO_PIN_2 | FL_GPIO_PIN_3 | FL_GPIO_PIN_4 |
                                 FL_GPIO_PIN_5;
            GPIO_InitStruct.mode = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull = FL_GPIO_BOTH_DISABLE;
            GPIO_InitStruct.remapPin = FL_GPIO_PINREMAP_FUNCTON0;
            GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
            (void)FL_GPIO_Init(GPIOD, &GPIO_InitStruct);
            break;
        case SPI2_BASE:
            GPIO_InitStruct.pin = FL_GPIO_PIN_8 | FL_GPIO_PIN_9 |
                                 FL_GPIO_PIN_10;
            GPIO_InitStruct.mode = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull = FL_GPIO_BOTH_DISABLE;
            GPIO_InitStruct.remapPin = FL_GPIO_PINREMAP_FUNCTON0;
            GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
            (void)FL_GPIO_Init(GPIOC, &GPIO_InitStruct);
        
            GPIO_InitStruct.pin = FL_GPIO_PIN_12;
            GPIO_InitStruct.remapPin = FL_GPIO_PINREMAP_FUNCTON1;
            GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
            (void)FL_GPIO_Init(GPIOC, &GPIO_InitStruct);
            break;
        case SPI3_BASE:
            GPIO_InitStruct.pin = FL_GPIO_PIN_6 | FL_GPIO_PIN_7 | FL_GPIO_PIN_8 |
                                 FL_GPIO_PIN_9;
            GPIO_InitStruct.mode = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.pull = FL_GPIO_BOTH_DISABLE;
            GPIO_InitStruct.remapPin = FL_GPIO_PINREMAP_FUNCTON0;
            GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
            (void)FL_GPIO_Init(GPIOG, &GPIO_InitStruct);
            break;
        default:
            break;
    }

    /*SSN输出控制*/
    SPI_InitStruct.softControl = FL_ENABLE;
    /*Master/Slave模式选择*/
    SPI_InitStruct.mode = FL_SPI_WORK_MODE_MASTER;
    /*Master模式波特率配置*/
    SPI_InitStruct.baudRate = SPI_BAUD;
    /*帧格式配置*/
    SPI_InitStruct.bitOrder = FL_SPI_BIT_ORDER_MSB_FIRST;
    /*数据位宽配置*/
    SPI_InitStruct.dataWidth = 8;
    /*时钟相位选择配置*/
    SPI_InitStruct.clockPhase = FL_SPI_PHASE_EDGE1;
    /*通信模式选择配置*/
    SPI_InitStruct.transferMode = FL_SPI_TRANSFER_MODE_FULL_DUPLEX;
    /*时钟极性选择配置*/
    SPI_InitStruct.clockPolarity = FL_SPI_POLARITY_NORMAL;

    FL_SPI_Init(SPIx, &SPI_InitStruct);

}


#define TIMEOUT_VAL     (0xFFFFFFFFUL)
/**
  * @brief  SPI读写数据
  * @param  data: 写数据
  * @retval data: 读数据
  */
uint32_t SpiWriteAndRead(SPI_Type* SPIx, uint32_t data)
{
    uint32_t timeout = 0;

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_LOW);

    FL_SPI_WriteTXBuff(SPIx, data);

    do
    {
        timeout++;
    } while(!FL_SPI_IsActiveFlag_TXBuffEmpty(SPIx) && timeout < TIMEOUT_VAL);

    timeout = 0;
    do
    {
        timeout++;
    } while(!FL_SPI_IsActiveFlag_RXBuffFull(SPIx) && timeout < TIMEOUT_VAL);

    data = FL_SPI_ReadRXBuff(SPIx);

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_HIGH);

    return data;
}

/**
  * @brief  SPI写数据
  * @param  data:   数据指针
  *         length: 数据长度
  * @retval void
  */
void SpiWriteData(SPI_Type* SPIx, uint8_t *data, uint16_t length)
{
    uint32_t timeout;
   FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_LOW);

    while(length--)
    {
        FL_SPI_WriteTXBuff(SPIx, *(data++));

        // 等待发送缓存空
        timeout = 0;
        do
        {
            timeout++;
        } while(!FL_SPI_IsActiveFlag_TXBuffEmpty(SPIx) && timeout < TIMEOUT_VAL);

        timeout = 0;
        do
        {
            timeout++;
        } while(!FL_SPI_IsActiveFlag_RXBuffFull(SPIx) && timeout < TIMEOUT_VAL);
        
        FL_SPI_ReadRXBuff(SPIx);
    }
     FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_HIGH);

}

/**
  * @brief  SPI读数据
  * @param  data:   数据指针
  *         length: 数据长度
  * @retval void
  */
void SpiReadData(SPI_Type* SPIx, uint8_t *data, uint16_t length)
{
    uint32_t timeout;

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_LOW);

    while(length--)
    {
        FL_SPI_WriteTXBuff(SPIx, 0x55);
        timeout = 0;
        do
        {
            timeout++;
        } while(!FL_SPI_IsActiveFlag_RXBuffFull(SPIx) && timeout < TIMEOUT_VAL);

        *data = FL_SPI_ReadRXBuff(SPIx);
        data++;
    }

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_HIGH);

}

/**
  * @brief  SPI DMA写数据
  * @param  dataAddr: 数据地址
  *         length:   数据长度
  * @retval void
  */
void SpiWriteDataDMA(SPI_Type* SPIx, uint32_t dataAddr, uint16_t length)
{
    uint32_t timeout;

    FL_SPI_ClearTXBuff(SPIx);

    FL_DMA_WriteTransmissionSize(DMA, length - 1, FL_DMA_CHANNEL_2);
    FL_DMA_WriteMemoryAddress(DMA, dataAddr, FL_DMA_CHANNEL_2);

    FL_DMA_ClearFlag_TransferComplete(DMA, FL_DMA_CHANNEL_2);

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_LOW);
    FL_DMA_EnableChannel(DMA, FL_DMA_CHANNEL_2);

    /* 等待DMA转换完成 */
    timeout = 0;
    do
    {
        timeout++;
    }while(!FL_DMA_IsActiveFlag_TransferComplete(DMA, FL_DMA_CHANNEL_2) && timeout < TIMEOUT_VAL);

    /* 等待发送完成 */
    timeout = 0;
    do
    {
        timeout++;
    } while(FL_SPI_IsActiveFlag_Busy(SPIx) && timeout < TIMEOUT_VAL);

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_HIGH);
}

/**
  * @brief  SPI DMA读数据
  * @param  dataAddr: 数据地址
  *         length:   数据长度
  * @retval void
  */
void SpiReadDataDMA(SPI_Type* SPIx, uint32_t dataAddr, uint16_t length)
{
    uint32_t timeout;

    FL_SPI_ClearRXBuff(SPIx);

    FL_DMA_WriteTransmissionSize(DMA, length - 1, FL_DMA_CHANNEL_2);
    FL_DMA_WriteMemoryAddress(DMA, dataAddr, FL_DMA_CHANNEL_2);

    FL_DMA_ClearFlag_TransferComplete(DMA, FL_DMA_CHANNEL_2);

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_LOW);
    FL_DMA_EnableChannel(DMA, FL_DMA_CHANNEL_2);

    /* 等待DMA转换完成 */
    timeout = 0;
    do
    {
        timeout++;
    }while(!FL_DMA_IsActiveFlag_TransferComplete(DMA, FL_DMA_CHANNEL_2) && timeout < TIMEOUT_VAL);

    /* 等待发送完成 */
    timeout = 0;
    do
    {
        timeout++;
    }while(FL_SPI_IsActiveFlag_Busy(SPIx) && timeout < TIMEOUT_VAL);

    FL_SPI_SetSSNPin(SPIx, FL_SPI_SSN_HIGH);
}

#if (SLEEP_SUPPORT == 1)
/**
  * @brief  SPI3 休眠前初始化
  * @param  void
  * @retval void
  */
void spi3_sleep_init(void)
{
    /* 关闭非唤醒外设 */
    FL_SPI_Disable(SPI3);
    
    /* 关闭非唤醒IO口 */
    CloseIO(GPIOG, FL_GPIO_PIN_6);
    CloseIO(GPIOG, FL_GPIO_PIN_7);
    CloseIO(GPIOG, FL_GPIO_PIN_8);
    CloseIO(GPIOG, FL_GPIO_PIN_9);
    
    /* 唤醒源配置 */
}

/**
  * @brief  SPI3 唤醒后初始化
  * @param  0-普通 1-作为本次唤醒的源
  * @retval void
  */
void spi3_awake_init(uint8_t awake_type)
{
    /* 打开非唤醒IO */
    AltFunIO( GPIOG, FL_GPIO_PIN_6,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );    //PG6;//SPI3_SSN
    AltFunIO( GPIOG, FL_GPIO_PIN_7,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );    //PG7;//SPI3_SCK
    AltFunIO( GPIOG, FL_GPIO_PIN_8,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );    //PG8;//SPI3_MISO
    AltFunIO( GPIOG, FL_GPIO_PIN_9,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );    //PG9;//SPI3_MOSI
    
    /* 任务初始化 */

    /* 唤醒源处理 */
    
    /* 非唤醒外设处理 */
    FL_SPI_Enable(SPI3);  
}

/**
  * @brief  SPI3 sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t spi3_is_sleep(void)
{
    if (!FL_SPI_IsActiveFlag_Busy(SPI3))
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
#endif
