/**
  ******************************************************************************
  * @attention
  * Copyright (c) 2023, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)
  * All rights reserved.
  *
  * Processor:                   FM33FG0xxA
  * http:                        http://www.fmdevelopers.com.cn/
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met
  *
  * 1. Redistributions of source code must retain the above copyright notice,
  *    this list of conditions and the following disclaimer.
  *
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *
  * 3. Neither the name of the copyright holder nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  *
  * 4. To provide the most up-to-date information, the revision of our documents
  *    on the World Wide Web will be the most Current. Your printed copy may be
  *    an earlier revision. To verify you have the latest information avaliable,
  *    refer to: http://www.fmdevelopers.com.cn/.    
  *
  * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED
    ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
    OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE
    ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE
    FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
    DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
    OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
    CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
    OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE
    USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

#include "tau.h"
#include "mf_config.h"

void TAU07_IRQHandler(void)
{
      /* 定时器溢出中断 */
    if(FL_TAU_IsEnabledIT_OC(TAU07) && FL_TAU_IsActiveFlag_OC(TAU07))
    {
        FL_TAU_ClearFlag_OC(TAU07);
    }
}

/** 
  * @brief  TAU输出比较初始化（PWM）
  * @param  void
  * @retval void
  */
void TAU_OC_Init(void)
{
    FL_TAU_InitTypeDef        InitStructer;
    FL_NVIC_ConfigTypeDef     InterruptConfigStruct;
    FL_GPIO_InitTypeDef       GPIO_InitStruct;
  
    /* 初始化引脚功能 */
    GPIO_InitStruct.pin           = FL_GPIO_PIN_9;
    GPIO_InitStruct.mode          = FL_GPIO_MODE_DIGITAL;
    GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
    GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
    GPIO_InitStruct.pull          = FL_GPIO_PULLUP_ENABLE;
    GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON2;
    FL_GPIO_Init(GPIOF, &GPIO_InitStruct);

    /* 初始化 TAU 配置参数 */
    InitStructer.mode               = FL_TAU_OPERATION_MODE_OC;      /* 定时器模式：输出比较 */
    InitStructer.counterSource      = FL_TAU_COUNTER_SOURCE_INTERNAL;/* 计数源配置：内部APBCLK */
    InitStructer.countEdge          = FL_TAU_COUNTER_EDGE_RISING;    /* 计数源边缘：上升沿 */
    InitStructer.prescaler          = 800-1;                          /* 计数源分频：80分频 */
    InitStructer.autoReload         = 10000-1;                       /* 自动重装载值：10000 */
    InitStructer.outputPolarity     = FL_TAU_OC_POLARITY_NORMAL;     /* 输出极性：正向 */
    InitStructer.compare            = 1000-1;                        /* 输出比较值：1000 */
    InitStructer.captureMode        = FL_TAU_IC_MODE_NORMAL;         /* 输入捕捉模式：自由模式 */
    InitStructer.captureSource      = FL_TAU_IC_SOURCE_GROUP0;       /* 输入捕捉源：GROUP0 */
    InitStructer.captureEdge        = FL_TAU_IC_EDGE_RISING;         /* 输入捕捉边缘：上升沿 */
    InitStructer.capturePrescaler   = 0;                             /* 输入捕捉源分频：不分频 */
    InitStructer.useOneShotCapture  = FL_DISABLE;                    /* 单次捕捉控制：连续捕捉 */
    InitStructer.useCaptureFilter   = FL_DISABLE;                    /* 输入捕捉滤波：关闭 */
    InitStructer.cascadeMode        = FL_TAU_CASCADE_MODE_DISABLE;   /* 级联模式：关闭 */
    InitStructer.slaveMode          = FL_TAU_SLAVE_MODE_TRIG_CNT;    /* 从机模式：触发递增模式 */
    InitStructer.slaveTriggerSource = FL_TAU_TRGI_GROUP0;            /* 从机触发源：GROUP0 */
  
    FL_TAU_Init(TAU07,&InitStructer);  /* 初始化TAU07 */
    
    FL_TAU_EnableOutput(TAU07);        /* 使能输出比较 */
    
    FL_TAU_ClearFlag_OC(TAU07);        /* 清除比较中断标志 */
    FL_TAU_EnableIT_OC(TAU07);         /* 使能比较中断 */
    

    /* 配置NVIC */
    InterruptConfigStruct.preemptPriority = TAU_PGL_IRQ_PRI;     
    FL_NVIC_Init(&InterruptConfigStruct, TAU_PGL_IRQn);
  
    /* 启动定时器 TAU07*/
    FL_TAU_Enable(TAU07); 
}

/* 能被10KHz整分频的频率 */
void set_pwm_para(uint32_t frequency, uint8_t duty)
{
    FL_TAU_InitTypeDef        InitStructer;
    uint32_t temp_prescaler, temp_compare;
    
    if (((frequency < 25) || (frequency > 10000)) ||
        duty > 100)
    {
        return;
    }
    temp_prescaler = 10000/frequency - 1;
    if (duty == 0)
    {
        InitStructer.outputPolarity     = FL_TAU_OC_POLARITY_INVERT;     /* 输出极性：负向 */        
        temp_compare = 0;        
    }
    else if (duty == 100)
    {
        InitStructer.outputPolarity     = FL_TAU_OC_POLARITY_NORMAL;     /* 输出极性：正向 */
        temp_compare = 0;
    }
    else
    {
        InitStructer.outputPolarity     = FL_TAU_OC_POLARITY_NORMAL;     /* 输出极性：正向 */
        temp_compare = (100 - duty) * __SYSTEM_CLOCK_10KHz_PRE/100 - 1;
    }
    
    FL_TAU_Disable(TAU07);

    /* 初始化 TAU 配置参数 */
    InitStructer.mode               = FL_TAU_OPERATION_MODE_OC;      /* 定时器模式：输出比较 */
    InitStructer.counterSource      = FL_TAU_COUNTER_SOURCE_INTERNAL;/* 计数源配置：内部APBCLK */
    InitStructer.countEdge          = FL_TAU_COUNTER_EDGE_RISING;    /* 计数源边缘：上升沿 */
    InitStructer.prescaler          = temp_prescaler;                /* 计数源分频：80分频 */
    InitStructer.autoReload         = __SYSTEM_CLOCK_10KHz_PRE-1;    /* 自动重装载值：10000 */
//    InitStructer.outputPolarity     = FL_TAU_OC_POLARITY_NORMAL;     /* 输出极性：正向 */
    InitStructer.compare            = temp_compare;                  /* 输出比较值：1000 */
    InitStructer.captureMode        = FL_TAU_IC_MODE_NORMAL;         /* 输入捕捉模式：自由模式 */
    InitStructer.captureSource      = FL_TAU_IC_SOURCE_GROUP0;       /* 输入捕捉源：GROUP0 */
    InitStructer.captureEdge        = FL_TAU_IC_EDGE_RISING;         /* 输入捕捉边缘：上升沿 */
    InitStructer.capturePrescaler   = 0;                             /* 输入捕捉源分频：不分频 */
    InitStructer.useOneShotCapture  = FL_DISABLE;                    /* 单次捕捉控制：连续捕捉 */
    InitStructer.useCaptureFilter   = FL_DISABLE;                    /* 输入捕捉滤波：关闭 */
    InitStructer.cascadeMode        = FL_TAU_CASCADE_MODE_DISABLE;   /* 级联模式：关闭 */
    InitStructer.slaveMode          = FL_TAU_SLAVE_MODE_TRIG_CNT;    /* 从机模式：触发递增模式 */
    InitStructer.slaveTriggerSource = FL_TAU_TRGI_GROUP0;            /* 从机触发源：GROUP0 */
  
    FL_TAU_Init(TAU07,&InitStructer);  /* 初始化TAU07 */
        
    FL_TAU_ClearFlag_OC(TAU07);        /* 清除比较中断标志 */
    FL_TAU_EnableIT_OC(TAU07);         /* 使能比较中断 */
  
    /* 启动定时器 TAU07*/
    FL_TAU_Enable(TAU07);    
}    

#if (SLEEP_SUPPORT == 1)
/**
  * @brief  TAU07休眠前初始化
  * @param  void
  * @retval void
  */
void tau07_sleep_init(void)
{
    /* 关闭非唤醒外设 */
    FL_TAU_Disable(TAU07);  /* 禁能定时器 */
    FL_TAU_DisableIT_OC(TAU07);         /* 禁能比较中断 */    
    FL_TAU_ClearFlag_OC(TAU07);        /* 清除比较中断标志 */
    Close_And_ClearIRQ(TAU_PGL_IRQn); 
    
    /* 关闭非唤醒IO口 */
    CloseIO(GPIOF, FL_GPIO_PIN_9);
    
    /* 唤醒源配置 */
}

/**
  * @brief  TAU07唤醒后初始化
  * @param  0-普通 1-作为本次唤醒的源
  * @retval void
  */
void tau07_awake_init(uint8_t awake_type)
{
    /* 打开非唤醒IO */
    AltFunIO( GPIOF, FL_GPIO_PIN_9,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_PULLUP_ENABLE, FL_GPIO_PINREMAP_FUNCTON2, FL_GPIO_DRIVESTRENGTH_X3 ); //PF9;//TAU PWM输出  
    
    /* 任务初始化 */

    /* 唤醒源处理 */
    
    /* 非唤醒外设处理 */
    FL_TAU_ClearFlag_OC(TAU07);        /* 清除比较中断标志 */
    FL_TAU_EnableIT_OC(TAU07);         /* 使能比较中断 */    
    NVIC_EnableIRQ(TAU_PGL_IRQn);
    FL_TAU_Enable(TAU07);  /* 使能定时器 */   
}

/**
  * @brief  TAU07 sleep判断
  * @param  void
  * @retval 1: Sleep
  *         0: Wake
  */
uint8_t tau07_is_sleep(void)
{
    if (1)//随时可进入sleep
    {
        return 1;
    }
    else
    {
        return 0;
    }
}
#endif
