 /**    
  *******************************************************************************************************   
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  

#include "main.h"
#include "uart.h"
#include "lin.h"

struct UARTOpStruct MasterUARTxOp;
struct UARTOpStruct SlaveUARTxOp;

/**
  * @brief  UART初始化
  * @param  UARTx: 串口选择
  * @retval void
  */
void Uartx_Init(UART_Type *UARTx)
{
    FL_GPIO_InitTypeDef GPIO_InitStruct = {0};
    FL_UART_InitTypeDef UART_InitStruct = {0};
    
    switch((uint32_t)UARTx)
    {
        case LIN_SLAVE_UART_BASE:
            /* Rx */
            GPIO_InitStruct.pin           = LIN_SLAVE_UART_RX_PIN;
            GPIO_InitStruct.mode          = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
            GPIO_InitStruct.pull          = FL_GPIO_PULLUP_ENABLE;
            GPIO_InitStruct.remapPin      = LIN_SLAVE_UART_RX_REMAP_FUNCTION;
            FL_GPIO_Init(LIN_SLAVE_UART_RX_GPIO, &GPIO_InitStruct);

            /* Tx */
            GPIO_InitStruct.pin           = LIN_SLAVE_UART_TX_PIN;
            GPIO_InitStruct.mode          = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
            GPIO_InitStruct.pull          = FL_GPIO_BOTH_DISABLE;
            GPIO_InitStruct.remapPin      = LIN_SLAVE_UART_TX_REMAP_FUNCTION;
            FL_GPIO_Init(LIN_SLAVE_UART_TX_GPIO, &GPIO_InitStruct);

            /* SLP引脚 */
            GPIO_InitStruct.pin           = FL_GPIO_PIN_9;
            GPIO_InitStruct.mode          = FL_GPIO_MODE_OUTPUT;
            GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
            GPIO_InitStruct.pull          = FL_GPIO_BOTH_DISABLE;
            GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON0;
            FL_GPIO_Init(GPIOE, &GPIO_InitStruct);
            
            GPIOE->DSET |= FL_GPIO_PIN_9;

            UART_InitStruct.clockSrc = FL_CMU_UART0_CLK_SOURCE_APBCLK;

            UART_InitStruct.baudRate            = LIN_BAUDRATE;               /* 波特率 */
            UART_InitStruct.dataWidth           = FL_UART_DATA_WIDTH_8B;      /* 数据位数 */
            UART_InitStruct.stopBits            = FL_UART_STOP_BIT_WIDTH_1B;  /* 停止位 */
            UART_InitStruct.parity              = FL_UART_PARITY_NONE;        /* 奇偶校验 */
            UART_InitStruct.transferDirection   = FL_UART_DIRECTION_TX;       /* 接收-发送使能 */
            FL_UART_Init(UARTx, &UART_InitStruct);
            
            NVIC_DisableIRQ(LIN_SLAVE_UART_IRQType);
            NVIC_SetPriority(LIN_SLAVE_UART_IRQType, LIN_SLAVE_UART_IRQ_PRI);             /* 中断优先级配置 */
            NVIC_EnableIRQ(LIN_SLAVE_UART_IRQType);
            break;
        case LIN_MASTER_UART_BASE:
            /* Rx */
            GPIO_InitStruct.pin           = LIN_MASTER_UART_RX_PIN;
            GPIO_InitStruct.mode          = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
            GPIO_InitStruct.pull          = FL_GPIO_PULLUP_ENABLE;
            GPIO_InitStruct.remapPin      = LIN_MASTER_UART_RX_REMAP_FUNCTION;
            FL_GPIO_Init(LIN_MASTER_UART_RX_GPIO, &GPIO_InitStruct);

            /* Tx */
            GPIO_InitStruct.pin           = LIN_MASTER_UART_TX_PIN;
            GPIO_InitStruct.mode          = FL_GPIO_MODE_DIGITAL;
            GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
            GPIO_InitStruct.pull          = FL_GPIO_BOTH_DISABLE;
            GPIO_InitStruct.remapPin      = LIN_MASTER_UART_TX_REMAP_FUNCTION;
            FL_GPIO_Init(LIN_MASTER_UART_TX_GPIO, &GPIO_InitStruct);
            
            /* SLP引脚 */
            GPIO_InitStruct.pin           = FL_GPIO_PIN_9;
            GPIO_InitStruct.mode          = FL_GPIO_MODE_OUTPUT;
            GPIO_InitStruct.outputType    = FL_GPIO_OUTPUT_PUSHPULL;
            GPIO_InitStruct.driveStrength = FL_GPIO_DRIVESTRENGTH_X3;
            GPIO_InitStruct.pull          = FL_GPIO_BOTH_DISABLE;
            GPIO_InitStruct.remapPin      = FL_GPIO_PINREMAP_FUNCTON0;
            FL_GPIO_Init(GPIOE, &GPIO_InitStruct);
            
            GPIOE->DSET |= FL_GPIO_PIN_9;

            UART_InitStruct.clockSrc = FL_CMU_UART0_CLK_SOURCE_APBCLK;

            UART_InitStruct.baudRate            = LIN_BAUDRATE;               /* 波特率 */
            UART_InitStruct.dataWidth           = FL_UART_DATA_WIDTH_8B;      /* 数据位数 */
            UART_InitStruct.stopBits            = FL_UART_STOP_BIT_WIDTH_1B;  /* 停止位 */
            UART_InitStruct.parity              = FL_UART_PARITY_NONE;        /* 奇偶校验 */
            UART_InitStruct.transferDirection   = FL_UART_DIRECTION_TX;       /* 接收-发送使能 */
            FL_UART_Init(UARTx, &UART_InitStruct);
            
            NVIC_DisableIRQ(LIN_MASTER_UART_IRQType);
            NVIC_SetPriority(LIN_MASTER_UART_IRQType, LIN_MASTER_UART_IRQ_PRI);             /* 中断优先级配置 */
            NVIC_EnableIRQ(LIN_MASTER_UART_IRQType);
            break;        
        default:
            break;
    }    
}
