 /**    
  *******************************************************************************************************   
  * @attention    
  * Copyright 2023 SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICRO.)
  *        
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met: 
  *    
  * 1. Redistributions of source code must retain the above copyright notice, 
  *    this list of conditions and the following disclaimer.
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,
  *    this list of conditions and the following disclaimer in the documentation
  *    and/or other materials provided with the distribution.
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors 
  *    may be used to endorse or promote products derived from this software without
  *    specific prior written permission.
  *    
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS “AS IS”
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   
  * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
  * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
  * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT   
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
  *******************************************************************************
  */  
  
#include "mf_config.h"
#include "fm33fg0xxa_fl.h"
#include "i2c.h"
#include "flash.h"
#include "lin.h"
#include <string.h>
#include "atim.h"
#include "canfd.h"

/* 校验寄存器 */
uint8_t CheckSysReg( __IO uint32_t *RegAddr, uint32_t Value )
{
    if( *RegAddr != Value ) 
    {
        *RegAddr = Value;
        return 1;
    }
    else
    {
        return 0;
    }
}

/* 查询NVIC寄存器对应向量号中断是否打开 */
/* 1 打开 */
/* 0 关闭 */
uint8_t CheckNvicIrqEn( IRQn_Type IRQn )
{
    if( 0 == ( NVIC->ISER[0U] & ((uint32_t)(1UL << (((uint32_t)(int32_t)IRQn) & 0x1FUL)))) )
        return 0;
    else
        return 1;
}

void Close_None_GPIO(void)
{
    /* 关闭芯片不可使用的IO(无对应引脚) */
    CloseIO( GPIOE, FL_GPIO_PIN_10 );
    CloseIO( GPIOE, FL_GPIO_PIN_11 );
    CloseIO( GPIOE, FL_GPIO_PIN_12 );
    CloseIO( GPIOE, FL_GPIO_PIN_13 );
    CloseIO( GPIOE, FL_GPIO_PIN_14 );
    CloseIO( GPIOE, FL_GPIO_PIN_15 );
    
    CloseIO( GPIOF, FL_GPIO_PIN_10 );
    CloseIO( GPIOF, FL_GPIO_PIN_11 );
    CloseIO( GPIOF, FL_GPIO_PIN_12 );
    CloseIO( GPIOF, FL_GPIO_PIN_13 );    
    
    CloseIO( GPIOI, FL_GPIO_PIN_9 );    
    CloseIO( GPIOI, FL_GPIO_PIN_10 );
    CloseIO( GPIOI, FL_GPIO_PIN_11 );
    CloseIO( GPIOI, FL_GPIO_PIN_12 );
    CloseIO( GPIOI, FL_GPIO_PIN_13 );
    CloseIO( GPIOI, FL_GPIO_PIN_14 );
    CloseIO( GPIOI, FL_GPIO_PIN_15 );   

    /* 关闭芯片未使用的IO(有对应引脚，应用未使用) */
    CloseIO( GPIOA, FL_GPIO_PIN_0 );
    CloseIO( GPIOA, FL_GPIO_PIN_1 );
/* CloseIO( GPIOA, FL_GPIO_PIN_2 ); */
/* CloseIO( GPIOA, FL_GPIO_PIN_3 ); */
    CloseIO( GPIOA, FL_GPIO_PIN_4 );
    CloseIO( GPIOA, FL_GPIO_PIN_5 ); 
    CloseIO( GPIOA, FL_GPIO_PIN_6 );
    CloseIO( GPIOA, FL_GPIO_PIN_7 );
    CloseIO( GPIOA, FL_GPIO_PIN_8 );
    CloseIO( GPIOA, FL_GPIO_PIN_9 );
    CloseIO( GPIOA, FL_GPIO_PIN_10 );
    CloseIO( GPIOA, FL_GPIO_PIN_11 );
    CloseIO( GPIOA, FL_GPIO_PIN_12 );
/* CloseIO( GPIOA, FL_GPIO_PIN_13 ); */
    CloseIO( GPIOA, FL_GPIO_PIN_14 );
    CloseIO( GPIOA, FL_GPIO_PIN_15 );

    CloseIO( GPIOB, FL_GPIO_PIN_0 );
    CloseIO( GPIOB, FL_GPIO_PIN_1 );
    CloseIO( GPIOB, FL_GPIO_PIN_2 );
    CloseIO( GPIOB, FL_GPIO_PIN_3 );
/* CloseIO( GPIOB, FL_GPIO_PIN_4 ); */
    CloseIO( GPIOB, FL_GPIO_PIN_5 ); 
/* CloseIO( GPIOB, FL_GPIO_PIN_6 ); */
    CloseIO( GPIOB, FL_GPIO_PIN_7 );
    CloseIO( GPIOB, FL_GPIO_PIN_8 );
    CloseIO( GPIOB, FL_GPIO_PIN_9 );
/* CloseIO( GPIOB, FL_GPIO_PIN_10 ); */
    CloseIO( GPIOB, FL_GPIO_PIN_11 );
    CloseIO( GPIOB, FL_GPIO_PIN_12 );
/* CloseIO( GPIOB, FL_GPIO_PIN_13 ); */
/* CloseIO( GPIOB, FL_GPIO_PIN_14 ); */
    CloseIO( GPIOB, FL_GPIO_PIN_15 ); 
    
    CloseIO( GPIOC, FL_GPIO_PIN_0 );
    CloseIO( GPIOC, FL_GPIO_PIN_1 );
/* CloseIO( GPIOC, FL_GPIO_PIN_2 ); */
/* CloseIO( GPIOC, FL_GPIO_PIN_3 ); */
    CloseIO( GPIOC, FL_GPIO_PIN_4 );
/* CloseIO( GPIOC, FL_GPIO_PIN_5 ); */
    CloseIO( GPIOC, FL_GPIO_PIN_6 );
    CloseIO( GPIOC, FL_GPIO_PIN_7 );
    CloseIO( GPIOC, FL_GPIO_PIN_8 );
    CloseIO( GPIOC, FL_GPIO_PIN_9 );
    CloseIO( GPIOC, FL_GPIO_PIN_10 );
    CloseIO( GPIOC, FL_GPIO_PIN_11 );
    CloseIO( GPIOC, FL_GPIO_PIN_12 );
    CloseIO( GPIOC, FL_GPIO_PIN_13 );
    CloseIO( GPIOC, FL_GPIO_PIN_14 );
    CloseIO( GPIOC, FL_GPIO_PIN_15 ); 

/* CloseIO( GPIOD, FL_GPIO_PIN_0 ); */
/* CloseIO( GPIOD, FL_GPIO_PIN_1 ); */
/* CloseIO( GPIOD, FL_GPIO_PIN_2 ); */
    CloseIO( GPIOD, FL_GPIO_PIN_3 );
    CloseIO( GPIOD, FL_GPIO_PIN_4 );
    CloseIO( GPIOD, FL_GPIO_PIN_5 ); 
    CloseIO( GPIOD, FL_GPIO_PIN_6 );
/* CloseIO( GPIOD, FL_GPIO_PIN_7 ); */ /* SWCLK */
/* CloseIO( GPIOD, FL_GPIO_PIN_8 ); */ /* SWIO */
/* CloseIO( GPIOD, FL_GPIO_PIN_9 ); */
/* CloseIO( GPIOD, FL_GPIO_PIN_10 ); */
    CloseIO( GPIOD, FL_GPIO_PIN_11 );
    CloseIO( GPIOD, FL_GPIO_PIN_12 );
    CloseIO( GPIOD, FL_GPIO_PIN_13 );
/* CloseIO( GPIOD, FL_GPIO_PIN_14 ); */
/* CloseIO( GPIOD, FL_GPIO_PIN_15 ); */
    
    CloseIO( GPIOE, FL_GPIO_PIN_0 );
    CloseIO( GPIOE, FL_GPIO_PIN_1 );
    CloseIO( GPIOE, FL_GPIO_PIN_2 );
    CloseIO( GPIOE, FL_GPIO_PIN_3 );
    CloseIO( GPIOE, FL_GPIO_PIN_4 );
    CloseIO( GPIOE, FL_GPIO_PIN_5 ); 
/* CloseIO( GPIOE, FL_GPIO_PIN_6 ); */
    CloseIO( GPIOE, FL_GPIO_PIN_7 );
    CloseIO( GPIOE, FL_GPIO_PIN_8 );
/* CloseIO( GPIOE, FL_GPIO_PIN_9 ); */

    CloseIO( GPIOF, FL_GPIO_PIN_0 );
    CloseIO( GPIOF, FL_GPIO_PIN_1 );
    CloseIO( GPIOF, FL_GPIO_PIN_2 );
    CloseIO( GPIOF, FL_GPIO_PIN_3 );
    CloseIO( GPIOF, FL_GPIO_PIN_4 );
    CloseIO( GPIOF, FL_GPIO_PIN_5 ); 
/* CloseIO( GPIOF, FL_GPIO_PIN_6 ); */
/* CloseIO( GPIOF, FL_GPIO_PIN_7 ); */
/* CloseIO( GPIOF, FL_GPIO_PIN_8 ); */
    CloseIO( GPIOF, FL_GPIO_PIN_9 );
    CloseIO( GPIOF, FL_GPIO_PIN_14 );
    CloseIO( GPIOF, FL_GPIO_PIN_15 );

    CloseIO( GPIOG, FL_GPIO_PIN_0 );
    CloseIO( GPIOG, FL_GPIO_PIN_1 );
    CloseIO( GPIOG, FL_GPIO_PIN_2 );
    CloseIO( GPIOG, FL_GPIO_PIN_3 );
    CloseIO( GPIOG, FL_GPIO_PIN_4 );
    CloseIO( GPIOG, FL_GPIO_PIN_5 ); 
/* CloseIO( GPIOG, FL_GPIO_PIN_6 ); */
/* CloseIO( GPIOG, FL_GPIO_PIN_7 ); */
/* CloseIO( GPIOG, FL_GPIO_PIN_8 ); */
/* CloseIO( GPIOG, FL_GPIO_PIN_9 ); */
    CloseIO( GPIOG, FL_GPIO_PIN_10 );
    CloseIO( GPIOG, FL_GPIO_PIN_11 );
    CloseIO( GPIOG, FL_GPIO_PIN_12 );
    CloseIO( GPIOG, FL_GPIO_PIN_13 );
    CloseIO( GPIOG, FL_GPIO_PIN_14 );
    CloseIO( GPIOG, FL_GPIO_PIN_15 ); 
    
    CloseIO( GPIOH, FL_GPIO_PIN_0 );
    CloseIO( GPIOH, FL_GPIO_PIN_1 );
    CloseIO( GPIOH, FL_GPIO_PIN_2 );
/* CloseIO( GPIOH, FL_GPIO_PIN_3 ); */
/* CloseIO( GPIOH, FL_GPIO_PIN_4 ); */
    CloseIO( GPIOH, FL_GPIO_PIN_5 ); 
    CloseIO( GPIOH, FL_GPIO_PIN_6 );
    CloseIO( GPIOH, FL_GPIO_PIN_7 );
    CloseIO( GPIOH, FL_GPIO_PIN_8 );
    CloseIO( GPIOH, FL_GPIO_PIN_9 );
/* CloseIO( GPIOH, FL_GPIO_PIN_10 ); */
/* CloseIO( GPIOH, FL_GPIO_PIN_11 ); */
/* CloseIO( GPIOH, FL_GPIO_PIN_12 ); */
    CloseIO( GPIOH, FL_GPIO_PIN_13 );
    CloseIO( GPIOH, FL_GPIO_PIN_14 );
/* CloseIO( GPIOH, FL_GPIO_PIN_15 ); */

    CloseIO( GPIOI, FL_GPIO_PIN_0 );
    CloseIO( GPIOI, FL_GPIO_PIN_1 );
    CloseIO( GPIOI, FL_GPIO_PIN_2 );
    CloseIO( GPIOI, FL_GPIO_PIN_3 );
    CloseIO( GPIOI, FL_GPIO_PIN_4 );
    CloseIO( GPIOI, FL_GPIO_PIN_5 ); 
    CloseIO( GPIOI, FL_GPIO_PIN_6 );
    CloseIO( GPIOI, FL_GPIO_PIN_7 );
    CloseIO( GPIOI, FL_GPIO_PIN_8 );
}

void Veri_Pad( void )
{    
    CheckSysReg( &GPIO->EXTIEDSA, 0xFFFFFFFF );
    CheckSysReg( &GPIO->EXTIEDSB, 0xFFFFFFFF );
    CheckSysReg( &GPIO->EXTIEDSC, 0xFFFFFFFF );
    CheckSysReg( &GPIO->EXTIEDSD, 0xFFFFFFFF );
    CheckSysReg( &GPIO->EXTIEDSE, 0xFFFFFFFF );
    CheckSysReg( &GPIO->EXTIEDSF, 0xFFFFFFFF );
    CheckSysReg( &GPIO->EXTIEDSG, 0xFFFFFFFF );
    CheckSysReg( &GPIO->EXTIEDSH, 0xFFFFFFFF );
    CheckSysReg( &GPIO->EXTIEDSI, 0xFFFFFFFF );
    CheckSysReg( &GPIO->FOUTSEL    , FL_GPIO_FOUT1_SELECT_XTHF_DIV64 | FL_GPIO_FOUT0_SELECT_AHBCLK_DIV64 );
    CheckSysReg( &GPIO->PINWKEN    , 0 );
    
    AltFunIO( GPIOA, FL_GPIO_PIN_2, FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_PULLUP_ENABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );     /* PA2 LinSlave UART0 Rx */
    AltFunIO( GPIOA, FL_GPIO_PIN_3, FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );      /* PA3 LinSlave UART0 Tx */
    OutputIO( GPIOA, FL_GPIO_PIN_13,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_DRIVESTRENGTH_X3 );    /* PA13 CAN STB引脚控制 */
    
    AltFunIO( GPIOB, FL_GPIO_PIN_4,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );      /* PB4 ATIM PWM输出 */
    OutputIO( GPIOB, FL_GPIO_PIN_6, FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_DRIVESTRENGTH_X3 );    /* PB6 指示灯 */
    AltFunIO( GPIOB, FL_GPIO_PIN_10, FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_PULLUP_ENABLE, FL_GPIO_PINREMAP_FUNCTON1, FL_GPIO_DRIVESTRENGTH_X3 );     /* PB10 GPTIM输入捕获 */
    AltFunIO( GPIOB, FL_GPIO_PIN_13, FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_PULLUP_ENABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );     /* PB13 LinMaster UART1 Rx */
    AltFunIO( GPIOB, FL_GPIO_PIN_14, FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );      /* PB14 LinMaster UART1 Tx */
    
    AnalogIO( GPIOC, FL_GPIO_PIN_2,  FL_GPIO_DRIVESTRENGTH_X3 );    /* PC2 XTHIN */
    AnalogIO( GPIOC, FL_GPIO_PIN_3,  FL_GPIO_DRIVESTRENGTH_X3 );    /* PC3 XTHOUT */
    AnalogIO( GPIOC, FL_GPIO_PIN_5,  FL_GPIO_DRIVESTRENGTH_X3 );    /* PC5 DAC */
    
    AnalogIO( GPIOD, FL_GPIO_PIN_0,  FL_GPIO_DRIVESTRENGTH_X3 );    /* PD0 ADC CH11 */
    AnalogIO( GPIOD, FL_GPIO_PIN_1,  FL_GPIO_DRIVESTRENGTH_X3 );    /* PD1 ADC CH12 */
    AnalogIO( GPIOD, FL_GPIO_PIN_2,  FL_GPIO_DRIVESTRENGTH_X3 );    /* PD2 ADC CH13 */
    AltFunIO( GPIOD, FL_GPIO_PIN_7,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_PULLUP_ENABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );     /* PD7 SWCLK */
    AltFunIO( GPIOD, FL_GPIO_PIN_8,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_PULLUP_ENABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );     /* PD8 SWIO */
    AnalogIO( GPIOD, FL_GPIO_PIN_9,  FL_GPIO_DRIVESTRENGTH_X3 );    /* PD9 ADC CH0 */
    AnalogIO( GPIOD, FL_GPIO_PIN_10, FL_GPIO_DRIVESTRENGTH_X3 );    /* PD10 ADC CH1 */
    AnalogIO( GPIOD, FL_GPIO_PIN_14, FL_GPIO_DRIVESTRENGTH_X3 );    /* PD14 ADC CH5 */
    AnalogIO( GPIOD, FL_GPIO_PIN_15, FL_GPIO_DRIVESTRENGTH_X3 );    /* PD15 ADC CH6 */
    
    InputIO ( GPIOE, FL_GPIO_PIN_6,  FL_GPIO_PULLUP_ENABLE );    /* PE6 按键 */
    AltFunIO( GPIOE, FL_GPIO_PIN_9,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON1, FL_GPIO_DRIVESTRENGTH_X3 );      /* PE9 CAN Tx */    

    AnalogIO( GPIOF, FL_GPIO_PIN_6,  FL_GPIO_DRIVESTRENGTH_X3 );    /* PF6 ADC CH7 */
    AnalogIO( GPIOF, FL_GPIO_PIN_7,  FL_GPIO_DRIVESTRENGTH_X3 );    /* PF7 ADC CH8 */
    OutputIO( GPIOF, FL_GPIO_PIN_8,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_DRIVESTRENGTH_X3 );    /* PF8 LIN SLP引脚控制 */

    AltFunIO( GPIOG, FL_GPIO_PIN_6,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );      /* PG6 SPI3_SSN */
    AltFunIO( GPIOG, FL_GPIO_PIN_7,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );      /* PG7 SPI3_SCK */
    AltFunIO( GPIOG, FL_GPIO_PIN_8,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );      /* PG8 SPI3_MISO */
    AltFunIO( GPIOG, FL_GPIO_PIN_9,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );      /* PG9 SPI3_MOSI */
    
    AltFunIO( GPIOH, FL_GPIO_PIN_3,  FL_GPIO_OUTPUT_OPENDRAIN, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );     /* PH3 I2C1 SCL */
    AltFunIO( GPIOH, FL_GPIO_PIN_4,  FL_GPIO_OUTPUT_OPENDRAIN, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );     /* PH4 I2C1 SDA */
    AltFunIO( GPIOH, FL_GPIO_PIN_10,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_PULLUP_ENABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );    /* PH10 CANFD0 Rx */
    AltFunIO( GPIOH, FL_GPIO_PIN_11,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );     /* PH11 CANFD0 Tx */
    OutputIO( GPIOH, FL_GPIO_PIN_12,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_DRIVESTRENGTH_X3 );    /* PH12 CANFD STB引脚控制 */
    AltFunIO( GPIOH, FL_GPIO_PIN_15,  FL_GPIO_OUTPUT_PUSHPULL, FL_GPIO_PULLUP_ENABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );    /* PH15 CAN Rx */
}

void Disable_I2C_IO(void)
{
    CloseIO( GPIOG, FL_GPIO_PIN_10 );        /* SCL */
    CloseIO( GPIOG, FL_GPIO_PIN_11 );        /* SDA */
}

void Enable_I2C_IO(void)
{
    AltFunIO( GPIOG, FL_GPIO_PIN_10,  FL_GPIO_OUTPUT_OPENDRAIN, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );    /* PG10 I2C1 SCL */
    AltFunIO( GPIOG, FL_GPIO_PIN_11,  FL_GPIO_OUTPUT_OPENDRAIN, FL_GPIO_BOTH_DISABLE, FL_GPIO_PINREMAP_FUNCTON0, FL_GPIO_DRIVESTRENGTH_X3 );    /* PG11 I2C1 SDA */
}

void Veri_CMU( void )               /* 时钟选择相关 */
{
    uint32_t counter = 0;
    uint32_t readystatus = 0;    
    
    CheckSysReg( &CMU->SYSCLKCR,  0x02000002 );
    CheckSysReg( &CMU->RCHFCR,  0x00000001 );
    if ((CMU->PLLCR & (0x00000033 | (PLLDB_VALUE << 16))) != (0x00000033 | (PLLDB_VALUE << 16)))
    {
        CMU->PLLCR = (0x00000033 | (PLLDB_VALUE << 16));
        
        do
        {
            readystatus = FL_CMU_IsActiveFlag_PLLReady();
            counter++;

            if ((counter % 100) == 0)
            {   /* 清狗 */
                FL_IWDT_ReloadCounter(IWDT);  
            }
        } while((readystatus != 0x1U) && (counter != 0xFFFFFFFFU));  /* 等待PLL建立 */
        
        /* 清除PLL失锁中断标志，必须放置在PLL成功锁定之后 */
        FL_CMU_ClearFlag_PLLLostLock();        
        /* 清除时钟选择错误标志 */
        FL_CMU_ClearFlag_SYSCSEWrong();
    }
    CheckSysReg( &CMU->XTLFCR,  0x0000E011 );       
    CheckSysReg( &CMU->LSCLKSEL,  0x00000101 );     
    CheckSysReg( &CMU->XTHFCR,  0x0000EA03 );       
    CheckSysReg( &CMU->IER,  0x00000000 );    
    CheckSysReg( &CMU->PCLKCR1,  0x000001BB );      
    CheckSysReg( &CMU->PCLKCR2,  0x00000510 );      
    CheckSysReg( &CMU->PCLKCR3,  0x02880304 );      
    CheckSysReg( &CMU->PCLKCR4,  0x000001A3 );      
    CheckSysReg( &CMU->OPCCR1,  0x00000830 );       
    CheckSysReg( &CMU->OPCCR2,  0x00000007 );       
    CheckSysReg( &CMU->OPCCR3,  0x0001B30F );       
    CheckSysReg( &CMU->AHBMCR,  0x80000000 );       
    CheckSysReg( &CMU->CFDCR,  0x00000090 );        
    CheckSysReg( &CMU->CFDER,  0x00000030 );   
    
    #ifdef __DEBUG
    CheckSysReg( &DBG->CR, 0x00010003 ); /* 仿真时运行定时器, 休眠模式下不关闭HCLK和FCLK */
    #else
    CheckSysReg( &DBG->CR, 0 );
    #endif
}

void Veri_FLASH( void )
{
    CheckSysReg( &FLASH->RDCR,  0x00000052 );
    CheckSysReg( &FLASH->OPTBR,  0x40000015 );
    /*
    CheckSysReg( &FLASH->CFLS_ECCR,  0x01000000 );
    CheckSysReg( &FLASH->DFLS_ECCR,  0x01000000 );
    CheckSysReg( &FLASH->RAM_ECCR,  0x01000000 );
    */
    CheckSysReg( &FLASH->HSRAM1_CFGR,  0x00000007 );
    CheckSysReg( &FLASH->HSRAM2_CFGR,  0x00000007 );    
}

void Veri_PMU( void )
{
    CheckSysReg( &PMU->CR,  0x00000000 );
    CheckSysReg( &PMU->WKTR,  0x00000001 );
    CheckSysReg( &PMU->IER,  0x00000000 );
    CheckSysReg( &PMU->BUFCR,  0x00000014 );
    CheckSysReg( &PMU->PTAT_CR,  0x00000001 );
    CheckSysReg( &PMU->CFDCK_CR,  0x00000000 );
}

void Veri_RMU( void )
{
    CheckSysReg( &RMU->PDRCR,  0x00000007 );
    CheckSysReg( &RMU->BORCR,  0x00000004 );
    CheckSysReg( &RMU->LKPCR,  0x00000000 );
}

void Veri_IWDT( void )
{
    CheckSysReg( &IWDT->IER,  0x00000000 );
    CheckSysReg( &IWDT->CR,  0x00000806 );
}

void Veri_SVD( void )
{
    CheckSysReg( &SVD->CFGR,  0x000000B8 );
    CheckSysReg( &SVD->IER,  0x00000000 );
    CheckSysReg( &SVD->VSR,  0x00000004 );
    CheckSysReg( &SVD->CR,  0x00000001 );
}

void Veri_LPTIM16( void )
{
    CheckSysReg( &LPTIM16->CFGR,  0x00000000 );
    CheckSysReg( &LPTIM16->CCSR,  0x00000000 );
    CheckSysReg( &LPTIM16->ARR,  320 - 1 );
    CheckSysReg( &LPTIM16->IER,  0x00000040 );
    CheckSysReg( &LPTIM16->CR,  0x00000001 );
    
    if( 0 == CheckNvicIrqEn(LPTIM_BSTIM_IRQn) )
    {
        NVIC_DisableIRQ(LPTIM_BSTIM_IRQn);
        NVIC_SetPriority(LPTIM_BSTIM_IRQn, LPTIM_BSTIM_IRQ_PRI); 
        NVIC_EnableIRQ(LPTIM_BSTIM_IRQn);
    }     
}

/* Lin Slave 应用 */
void Veri_UART0( void )
{
    CheckSysReg( &UART0->CSR,  0x00000043 );/* TODO 需要验证是否维持不变 */
    if ((UART0->IER & (BIT25 | BIT8)) != (BIT25 | BIT8))
    {
        UART0->IER |= (BIT25 | BIT8);
    }
    CheckSysReg( &UART0->MCR,  0x00000003 );
    
    if( 0 == CheckNvicIrqEn(UART0_IRQn) )
    {
        NVIC_DisableIRQ(UART0_IRQn);
        NVIC_SetPriority(UART0_IRQn, LIN_SLAVE_UART_IRQ_PRI);             /* 中断优先级配置 */
        NVIC_EnableIRQ(UART0_IRQn);    
    }
}

/* Lin Master 应用 */
void Veri_UART1( void )
{
    CheckSysReg( &UART1->CSR,  0x00000043 );
    if ((UART1->IER & (BIT25 | BIT8)) != (BIT25 | BIT8))
    {
        UART1->IER |= (BIT25 | BIT8);
    }
    CheckSysReg( &UART1->MCR,  0x00000001 ); 
    
    if( 0 == CheckNvicIrqEn(UART1_IRQn) )
    {
        NVIC_DisableIRQ(UART1_IRQn);
        NVIC_SetPriority(UART1_IRQn, LIN_MASTER_UART_IRQ_PRI);             /* 中断优先级配置 */
        NVIC_EnableIRQ(UART1_IRQn);    
    }    
}

void Veri_LPTIM32( void )
{
    CheckSysReg( &LPTIM16->CFGR,  0x00000000 );
    CheckSysReg( &LPTIM16->CCSR,  0x00000000 );
    CheckSysReg( &LPTIM16->ARR,  TIMER_1MS_LOAD - 1 );
    CheckSysReg( &LPTIM16->IER,  0x00000040 );
    CheckSysReg( &LPTIM16->CR,  0x00000001 );
    
    if( 0 == CheckNvicIrqEn(LPTIM_BSTIM_IRQn) )
    {
        NVIC_DisableIRQ(LPTIM_BSTIM_IRQn);
        NVIC_SetPriority(LPTIM_BSTIM_IRQn, LPTIM_BSTIM_IRQ_PRI); 
        NVIC_EnableIRQ(LPTIM_BSTIM_IRQn);
    }     
}

/* Lin Master和Slave共用 */
void Veri_TAU13( void )
{
    CheckSysReg( TAU13->CFGR,  0x00000000 );
    CheckSysReg( TAU13->MDR,  0x00000000 );
    CheckSysReg( TAU13->ARR,  TIMER_1US_LOAD * TIME_BASE_PERIOD - 1 );
    CheckSysReg( TAU13->IER,  0x00000001 );
    CheckSysReg( TAU13->CR,  0x00000008 );
    
    if( 0 == CheckNvicIrqEn(TAU_PGL_IRQn) )
    {
        NVIC_DisableIRQ(TAU_PGL_IRQn);
        NVIC_SetPriority(TAU_PGL_IRQn, TAU_PGL_IRQ_PRI);
        NVIC_EnableIRQ(TAU_PGL_IRQn);   
    }    
}

/* EEprom存储 应用 */
void Veri_I2C1( void )
{
    CheckSysReg( &I2C1->MSPIER,  0x00000000 );
    CheckSysReg( &I2C1->MSPBGR,  ((I2C_BRG << 16) | I2C_BRG) );
    CheckSysReg( &I2C1->MSPTCR,  0x0000008F );
    CheckSysReg( &I2C1->MSPCFGR,  0x00000001 );
}

/* 测试SPI3 应用 */
void Veri_SPI3( void )
{
    CheckSysReg( &SPI3->CR1,  (0x00000100 | SPI_BAUD) );
    CheckSysReg( &SPI3->IER,  0x00000000 );
    CheckSysReg( &SPI3->CR2,  0x00000057 );
    CheckSysReg( &SPI3->I2SCR,  0x00000000 );
    CheckSysReg( &SPI3->I2SPR,  0x00000000 );
}

/* FSCAN 应用 */
void Veri_FSCAN( void )
{
    /* 可以直接写的寄存器 */
    CheckSysReg( &CAN->RX_IER,  0x00001080 );
    if ((CAN->TXB_CR & 0x7) != 0)
    {
        CAN->TXB_CR &= ~0x7;
    }
    CheckSysReg( &CAN->AFCR,  0x00000003 );
    CheckSysReg( &CAN->AFCFGR,  0x00000000 );
    CheckSysReg( &CAN->AFMR1,  0x000007FF );
    CheckSysReg( &CAN->AFIR1,  0x0000003C );
    CheckSysReg( &CAN->AFMR2,  0x000007FF );
    CheckSysReg( &CAN->AFIR2,  0x00000017 );
    
    /* 需要先进入配置模式才可写的寄存器 */
    if ((CAN->MSR != 0x00003C00)  ||
        (CAN->BRPR != 0x00000007) ||
        (CAN->BTR != 0x0000013A))
    {
        CAN->CR = 0;
        CheckSysReg( &CAN->MSR,  0x00003C00 );
        CheckSysReg( &CAN->BRPR,  0x00000007 );
        CheckSysReg( &CAN->BTR,  0x0000013A );
        CAN->CR = 2;
    }
    
    if( 0 == CheckNvicIrqEn(FSCAN_IRQn) )
    {
        NVIC_DisableIRQ(FSCAN_IRQn);
        NVIC_SetPriority(FSCAN_IRQn, CAN_IRQ_PRI); 
        NVIC_EnableIRQ(FSCAN_IRQn);
    }    
}

/* CANFD 应用 */
void Veri_CANFD0( void )
{
    /* 判断寄存器配置 */
    if ((CANFD0->CFDC0NCFG != 0x06140807) ||
        ((CANFD0->CFDC0CTR & 0x1F00)!= 0x1F00) ||
        (CANFD0->CFDGCFG != 0x00000000) ||
        (CANFD0->CFDGCTR != 0x00000000) ||
        (CANFD0->CFDGAFLECTR != 0x00000001) ||
        (CANFD0->CFDGAFLCFG != 0x00020000) ||
        (CANFD0->CFDRMNB != 0x00000001) ||
        (CANFD0->CFDRMIEC != 0x00000001) ||
        (CANFD0->CFDRFCC[0] != 0x0000F203) ||
        (CANFD0->CFDRFCC[1] != 0x0000F203) ||
        (CANFD0->CFDCFCC != 0x0043F003) ||
        (CANFD0->CFDTMIEC != 0x00000000) ||
        (CANFD0->CFDTXQCC != 0x00000000) ||
        (CANFD0->CFDTHLCC != 0x00000000) ||
        (CANFD0->CFDGTSTCFG != 0x00000000) ||
        (CANFD0->CFDGTSTCTR != 0x00000000) ||
        (CANFD0->CFDGFDCFG != 0x00000000) ||
        (CANFD0->CFDGLOCKK != 0x00000000) ||
        (CANFD0->CFDGAFLIGNENT != 0x00000000) ||
        (CANFD0->CFDGAFLIGNCTR != 0x00000000) ||
        (CANFD0->CFDGPFLECTR != 0x00000000) ||
        (CANFD0->CFDGPFLCFG != 0x00000000) ||
        (CANFD0->CFDGRSTC != 0x00000000) ||
        (CANFD0->CFDC0DCFG != 0x02030A07) ||
        (CANFD0->CFDC0FDCFG != 0x000C0600) ||
        (CANFD0->CFDC0FDCTR != 0x00000000))
    {
       CANFD_Init(CANFD0);
    }
    
    if( 0 == CheckNvicIrqEn(CANFD0_IRQn) )
    {
        NVIC_DisableIRQ(CANFD0_IRQn);
        NVIC_SetPriority(CANFD0_IRQn, CANFD0_IRQ_PRI); 
        NVIC_EnableIRQ(CANFD0_IRQn);
    }    
}

/* GPTIM0捕获频率和占空比 应用 */
void Veri_GPTIM0( void )
{
    CheckSysReg( &GPTIM0->CR2,  0x00000000 );
    CheckSysReg( &GPTIM0->SMCR,  0x00000054 );
    CheckSysReg( &GPTIM0->DIER,  0x00000006 );
    CheckSysReg( &GPTIM0->EGR,  0x00000000 );
    CheckSysReg( &GPTIM0->CCMR1,  0x00000201 );
    CheckSysReg( &GPTIM0->CCMR2,  0x00000000 );
    CheckSysReg( &GPTIM0->CCER,  0x00000031 );
    CheckSysReg( &GPTIM0->PSC,  __SYSTEM_CLOCK_1MHz_PRE - 1 );
    CheckSysReg( &GPTIM0->ARR,  GPTIM0_AUTORELOAD_VALUE - 1 );
    CheckSysReg( &GPTIM0->CR1,  0x00000081 );
}

/* ATIM PWM输出 应用 */
void Veri_ATIM( void )
{
    CheckSysReg( &ATIM->CR2,  0x00000300 );
    CheckSysReg( &ATIM->SMCR,  0x00000000 );
    CheckSysReg( &ATIM->DIER,  0x00000000 );
    CheckSysReg( &ATIM->EGR,  0x00000000 );
    CheckSysReg( &ATIM->CCMR1,  0x00000060 );
    CheckSysReg( &ATIM->CCMR2,  0x00000000 );
    CheckSysReg( &ATIM->CCER,  0x00000001 );
    CheckSysReg( &ATIM->ARR,  __SYSTEM_CLOCK_10KHz_PRE-1 );
    CheckSysReg( &ATIM->RCR,  0x00000000 );
    CheckSysReg( &ATIM->CCR2,  0x00000000 );
    CheckSysReg( &ATIM->CCR3,  0x00000000 );
    CheckSysReg( &ATIM->CCR4,  0x00000000 );
    CheckSysReg( &ATIM->BDTR,  0x0000A064 );
    CheckSysReg( &ATIM->DCR,  0x00000000 );
/* CheckSysReg( &ATIM->DMAR,  0x00000000 ); */ /* 与CR1联动 */
    CheckSysReg( &ATIM->BKCR,  0x00000300 );
    CheckSysReg( &ATIM->CR1,  0x00000081 );
}

/* ADC序列转换 应用 */
void Veri_ADC( void )
{
    CheckSysReg( &ADC->CR2,  0x00000000 );
    CheckSysReg( &ADC->CFGR1,  0x00000000 );
    CheckSysReg( &ADC->CFGR2,  0x000005F1 );
    CheckSysReg( &ADC->SMTR,  0x000000BB );
    CheckSysReg( &ADC->ECHER,  0x0000B9E3 );
    CheckSysReg( &ADC->ICHER,  0x00000000 );
    CheckSysReg( &ADC->CR1,  0x00000001 );
    
    if( 0 == CheckNvicIrqEn(ADC_IRQn) )
    {
        NVIC_DisableIRQ(ADC_IRQn);
        NVIC_SetPriority(ADC_IRQn, ADC_IRQ_PRI); 
        NVIC_EnableIRQ(ADC_IRQn);
    }    
}

void Veri_DMA( void )
{
    CheckSysReg( &DMA->CH4CR,  0x0013281F );
/* CheckSysReg( &DMA->CH4MAR,  0x00000000 ); */ /* TODO 存储地址，根据Buffer获取 */
    CheckSysReg( &DMA->GCR,  0x00000001 );
    
    if( 0 == CheckNvicIrqEn(DMA_IRQn) )
    {
        NVIC_DisableIRQ(DMA_IRQn);
        NVIC_SetPriority(DMA_IRQn, DMA_IRQ_PRI); 
        NVIC_EnableIRQ(DMA_IRQn);
    }    
}

void Veri_BSTIM32( void )
{
    CheckSysReg( &BSTIM32->CR2,  0x00000000 );
    CheckSysReg( &BSTIM32->IER,  0x00000001 );
    CheckSysReg( &BSTIM32->EGR,  0x00000000 );
    CheckSysReg( &BSTIM32->PSC,  __SYSTEM_CLOCK_1MHz_PRE - 1 );
    CheckSysReg( &BSTIM32->ARR,  BSTIM32_AUTORELOAD_VALUE - 1 );
    CheckSysReg( &BSTIM32->CR1,  0x00000081 );
    
    if( 0 == CheckNvicIrqEn(LPTIM_BSTIM_IRQn) )
    {
        NVIC_DisableIRQ(LPTIM_BSTIM_IRQn);
        NVIC_SetPriority(LPTIM_BSTIM_IRQn, LPTIM_BSTIM_IRQ_PRI); 
        NVIC_EnableIRQ(LPTIM_BSTIM_IRQn);
    }    
}

void Veri_BSTIM16( void )
{
    CheckSysReg( &BSTIM16->CR2,  0x00000020 );
/* CheckSysReg( &BSTIM16->IER,  0x00000001 );*/ /* 应用会改变 */
    CheckSysReg( &BSTIM16->EGR,  0x00000000 );
    CheckSysReg( &BSTIM16->PSC,  __SYSTEM_CLOCK_1MHz_PRE - 1 );
    CheckSysReg( &BSTIM16->ARR,  BSTIM16_AUTORELOAD_VALUE - 1 );
    if ((BSTIM16->CR1 & BIT7) != BIT7)
    {
        BSTIM16->CR1 |= BIT7;
    }
    
    if( 0 == CheckNvicIrqEn(LPTIM_BSTIM_IRQn) )
    {
        NVIC_DisableIRQ(LPTIM_BSTIM_IRQn);
        NVIC_SetPriority(LPTIM_BSTIM_IRQn, LPTIM_BSTIM_IRQ_PRI); 
        NVIC_EnableIRQ(LPTIM_BSTIM_IRQn);
    }    
}

/* DAC测试 应用 */
void Veri_DAC( void )
{
    CheckSysReg( &DAC->CR2,  0x00000000 );
    CheckSysReg( &DAC->CFGR,  0x00000001 );
    CheckSysReg( &DAC->IER,  0x00000000 );
    CheckSysReg( &DAC->CR1,  0x00000001 );
}

void Veri_Sys_Reg( void )
{
    Veri_Pad();
    Veri_CMU();
    Veri_FLASH();
    Veri_PMU();
    Veri_RMU();
    Veri_IWDT();
    Veri_SVD();
    Veri_LPTIM16();
    Veri_UART0();
    Veri_UART1();
    Veri_LPTIM32();
    Veri_TAU13();
    Veri_I2C1();
    Veri_SPI3();
    Veri_FSCAN();
    Veri_CANFD0();
    Veri_GPTIM0();
    Veri_ATIM();
    Veri_ADC();
    Veri_DMA();
    Veri_BSTIM32();
    Veri_BSTIM16();
    Veri_DAC();
}

unsigned short Crc_16Bits( uint8_t *buf, uint8_t len ) 
{
    uint8_t i, j;
    unsigned short current,CRC16;
            
    CRC16 = 0x0000;

    for( i=0; i<len; i++ )
    {
        current = buf[i];
        for (j = 0; j < 8; j++) 
        { 
            if ((CRC16 ^ current) & 0x0001)
                CRC16 = (CRC16 >> 1) ^ 0x8408;
            else 
                CRC16 >>= 1; 
            current >>= 1;            
        }
    }
    
    return CRC16;
}

void Crc_16Bits_Add2( uint8_t *buf, uint8_t len ) 
{
    unsigned short Result;
    
    Result  = Crc_16Bits( buf, len );
    buf[len] = Result;        /* 低字节 */
    buf[len+1] = Result>>8;        /* 高字节 */
}


uint8_t Crc_Check( uint8_t *buf, uint8_t len ) 
{
    uint8_t i;

    for( i=0; i<len; i++ )
    {
        if ( buf[i] != 0 ) break;
    }
    if( i == len ) return 1;        /* 所有数据均为0,认为校验错 */
    
    if( Crc_16Bits( buf, len ) ) return 1;    /* crc校验错误 */
    
    return 0;                /* 校验正确 */
}

uint8_t Verify_EEprom_para(uint8_t Page ,uint8_t *Buf, uint8_t Len )/* EEprom一托二数据校验函数 */
{
    if( Crc_Check( Buf, Len+2 ) == 0 ) return 0;    /* ram参数crc正确 */
    
    if( I2C_Read_Eeprom(I2C1, Page, 0x00, Buf, Len ) == 0 ) return 0;/* 已经从数据ee主恢复 */
    
    if( I2C_Read_Eeprom(I2C1, Page, 0x20, Buf, Len ) != 0 ) return 1;/* 数据全错 */
    
    I2C_Write_Eeprom(I2C1, Page, 0x00, Buf, Len );    /* 从数据ee备份恢复,同时写主 */
    
    return 0;
}

uint8_t Verify_DataFlash_para(uint32_t Page, uint16_t offset, uint8_t *Buf, uint16_t Len)/* DataFlash一托二数据校验函数 */
{
    if( Crc_Check( Buf, Len+2 ) == 0 ) return 0;    /* ram参数crc正确 */
    
    if( Read_DataFlash(Page, offset, Buf, Len ) == 0 ) return 0;/* 已经从数据DataFlash主恢复 */
    
    if( Read_DataFlash(Page + DATA_FLASH_BKUP_PAGE, offset, Buf, Len ) != 0 ) return 1;/* 数据全错 */
    
    Write_DataFlash(Page, offset, Buf, Len );    /* 从数据DataFlash备份恢复,同时写主 */
    
    return 0;
}

void Ctrldata_EEProm_Verify()        /* 控制数据校验 */
{
    app_struct_type temp_app_struct;
    uint32_t primask;    
        
    /* 中断全局变量，竞态保护 */
    primask = __get_PRIMASK();
    __disable_irq();
    memcpy((void *)&temp_app_struct, (void *)&g_app_struct, sizeof(app_struct_type));
    __set_PRIMASK(primask);
    
    if( Verify_EEprom_para( ee_ctrldata_page, (uint8_t *)temp_app_struct.ctrl_data.data, ee_ctrldata_lenth ) )
    {
        memset( (void *)temp_app_struct.ctrl_data.data, 0x00, 8 );/* 控制数据默认值 */
    }
    
    /* 中断全局变量，竞态保护 */
    primask = __get_PRIMASK();
    __disable_irq();
    memcpy((void *)&g_app_struct, (void *)&temp_app_struct, sizeof(app_struct_type));
    __set_PRIMASK(primask);
}

void Ctrldata_DataFlash_Verify()        /* 控制数据校验 */
{
    app_struct_type temp_app_struct;
    uint32_t primask;    
        
    /* 中断全局变量，竞态保护 */
    primask = __get_PRIMASK();
    __disable_irq();
    memcpy((void *)&temp_app_struct, (void *)&g_app_struct, sizeof(app_struct_type));
    __set_PRIMASK(primask);
    
    if( Verify_DataFlash_para( dataflash_ctrldata_page, dataflash_ctrldata_offset, (uint8_t *)temp_app_struct.ctrl_data.data, dataflash_ctrldata_lenth ) )
    {
        memset( (void *)temp_app_struct.ctrl_data.data, 0x00, 8 );/* 控制数据默认值 */
    }
    
    /* 中断全局变量，竞态保护 */
    primask = __get_PRIMASK();
    __disable_irq();
    memcpy((void *)&g_app_struct, (void *)&temp_app_struct, sizeof(app_struct_type));
    __set_PRIMASK(primask);
}
